<?xml version="1.0" encoding="UTF-8" ?>
<!--  
 | Composite DITA topics to HTML
 |
 | (C) Copyright IBM Corporation 2001, 2002, 2003. All Rights Reserved.
 | This file is part of the DITA package on IBM's developerWorks site.
 | See license.txt for disclaimers.
 +
 | updates:
 *-->

<!-- write technique adapted from Norman Walsh's DocBook XSLT
     first instance technique adapted from Jeni Tennison and Steve Muench -->

<xsl:stylesheet version="1.0" 
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:html="http://www.w3.org/1999/xhtml"
                xmlns:saxon="http://icl.com/saxon"
                xmlns:lxslt="http://xml.apache.org/xslt"
                xmlns:xalanredirect="org.apache.xalan.xslt.extensions.Redirect"
                xmlns:exsl="http://exslt.org/common"
                extension-element-prefixes="saxon xalanredirect lxslt exsl"
				exclude-result-prefixes="html">

<!-- stylesheet imports -->

<xsl:import href="topic2html.xsl"/>
<xsl:import href="task2html.xsl"/>
<xsl:import href="ref2html.xsl"/>

<xsl:param name="outputdir">.</xsl:param>
<xsl:param name="toctype">plain</xsl:param>

<xsl:output
    method="xml"
    encoding="utf-8"
    indent="no"
    doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"
    doctype-public="-//W3C//DTD XHTML 1.0 Transitional//EN"
/>

<xsl:key name="topicref"
         match="*[contains(@class, ' map/topicref ')]"
         use="@href"/>

<xsl:template match="/">
  <xsl:apply-templates select="*[contains(@class,' map/map ')]"/>
</xsl:template>

<xsl:template match="*[contains(@class,' map/map ')]">
  <xsl:choose>
  <!-- other toc types such as Microsoft HTML Help and JavaHelp could
       be added -->
  <xsl:when test="$toctype = 'eclipse'">
    <toc label="{@title}">
      <xsl:apply-templates select="*[contains(@class,' map/topicref ')]"/>
    </toc>
  </xsl:when>
  <xsl:when test="$toctype = 'plain'">
    <h1>
      <xsl:value-of select="@title"/>
    </h1>
    <ul>
      <xsl:apply-templates select="*[contains(@class,' map/topicref ')]"/>
    </ul>
  </xsl:when>
  <xsl:otherwise>
    <xsl:message>
      <xsl:text>Unknown TOC type parameter </xsl:text>
      <xsl:value-of select="$toctype"/>
    </xsl:message>
  </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="*[contains(@class,' map/topicref ')]">
  <xsl:variable name="infile" select="@href"/>
  <xsl:variable name="outfile"
      select="concat(substring-before($infile, '.xml'), '.html')"/>
  <xsl:variable name="isFirst"
      select="generate-id(.) = generate-id(key('topicref', $infile)[1])"/>
  <xsl:variable name="root" select="/"/>
  <xsl:variable name="fileTitle">
    <xsl:if test="$isFirst or not(boolean(@navtitle))">
      <xsl:apply-templates select="document($infile, $root)" mode="topic">
        <xsl:with-param name="outfile" select="$outfile"/>
        <xsl:with-param name="isFirst" select="$isFirst"/>
      </xsl:apply-templates>
    </xsl:if>
  </xsl:variable>
  <xsl:variable name="title">
    <xsl:choose>
    <xsl:when test="@navtitle">
      <xsl:value-of select="@navtitle"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$fileTitle"/>
    </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="subtopicNodes"
    select="*[contains(@class,' map/topicref ')]"/>
  <xsl:choose>
  <xsl:when test="$toctype = 'eclipse'">
    <topicref href="{$outfile}" label="{$title}">
    <xsl:if test="$subtopicNodes">
      <xsl:apply-templates select="$subtopicNodes"/>
    </xsl:if>
    </topicref>
  </xsl:when>
  <xsl:when test="$toctype = 'plain'">
    <li>
      <a href="{$outfile}">
        <xsl:value-of select="$title"/>
      </a>
      <xsl:if test="$subtopicNodes">
        <ul>
          <xsl:apply-templates select="$subtopicNodes"/>
        </ul>
      </xsl:if>
    </li>
  </xsl:when>
  </xsl:choose>
</xsl:template>

<xsl:template match="/" mode="topic">
  <xsl:param name="isFirst" select="false()"/>
  <xsl:param name="outfile"/>
  <!-- output only the first reference to a file -->
  <xsl:if test="$isFirst">
    <xsl:variable name="page">
      <xsl:apply-templates mode="toplevel"/>
    </xsl:variable>
    <xsl:message>
      <xsl:value-of select="$outfile"/>
    </xsl:message>
    <xsl:call-template name="write-page">
      <xsl:with-param name="outfile" select="$outfile"/>
      <xsl:with-param name="page" select="$page"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:value-of select="*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/title ')]"/>
</xsl:template>

<xsl:template name="write-page">
  <xsl:param name="outfile"/>
  <xsl:param name="page"/>
  <xsl:variable name="filename" select="concat($outputdir, '/', $outfile)"/>
  <xsl:choose>
  <xsl:when test="element-available('saxon:output')">
    <saxon:output href="{$filename}"
                  method="html"
                  encoding="UTF-8"
                  indent="no"
                  saxon:character-representation="entity;decimal">
      <xsl:copy-of select="$page"/>
    </saxon:output>
  </xsl:when>
<!-- UNTESTED
  <xsl:when test="element-available('exsl:document')">
    <exsl:document href="{$filename}"
                   method="html"
                   encoding="UTF-8"
                   indent="no">
      <xsl:copy-of select="$page"/>
    </exsl:document>
  </xsl:when>
-->
  <xsl:when test="element-available('xalanredirect:write')">
    <xalanredirect:write file="{$filename}">
      <xsl:copy-of select="$page"/>
    </xalanredirect:write>
  </xsl:when>
  <xsl:otherwise>
    <xsl:message terminate="yes">
      <xsl:text>Cannot generate files with XSLT processor from </xsl:text>
      <xsl:value-of select="system-property('xsl:vendor')"/>
    </xsl:message>
  </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="last-value">
  <xsl:param name="value"/>
  <xsl:param name="separator" select="' '"/>
  <xsl:choose>
  <xsl:when test="not(string-length($value) &gt; 0)"/>
  <xsl:when test="not(contains($value, $separator))">
    <xsl:value-of select="$value"/>
  </xsl:when>
  <xsl:otherwise>
    <xsl:variable name="after" select="substring-after($value, $separator)"/>
    <xsl:variable name="result">
      <xsl:if test="string-length($after) &gt; 0">
        <xsl:call-template name="last-value">
          <xsl:with-param name="value"     select="$after"/>
          <xsl:with-param name="separator" select="$separator"/>
        </xsl:call-template>
      </xsl:if>
    </xsl:variable>
    <xsl:choose>
    <xsl:when test="string-length($result) &gt; 0">
      <xsl:value-of select="$result"/>
    </xsl:when>
    <xsl:when test="not(starts-with($value, $separator))">
      <xsl:variable name="before"
          select="substring-before($value, $separator)"/>
      <xsl:if test="string-length($before) &gt; 0">
        <xsl:value-of select="$before"/>
      </xsl:if>
    </xsl:when>
    </xsl:choose>  
  </xsl:otherwise>
  </xsl:choose>  
</xsl:template>

</xsl:stylesheet>
