<?xml version="1.0" encoding="UTF-8" ?>
<!--  topic2fo.xsl
 | DITA topic to FO; "single topic to single printed 'chapter'"-level view
 |
 | (C) Copyright IBM Corporation 2001, 2002, 2003. All Rights Reserved.
 | This file is part of the DITA package on IBM's developerWorks site.
 | See license.txt for disclaimers.
 +
 | updates:
 |   marker
 |   relcolwidth for simpletable
 |
 *-->


<!-- TBD:
work on rel-links and domains for FO

1.   Support @relcolwidth.
     A relative value to specify the width of a column in relationship to the width of the
     other columns for print output. The values are totaled and made a percent. For example: 
       relcolwidth="1* 2* 3*"
     causes widths of 16.7%, 33.3%, and 66.7%. 
       relcolwidth="90* 150*"
     causes width of 37.5% and 62.5%.

     No relcolwidth means "equally balance the table columns within the maximum table width."
     It is an error for the number of widths to not match the number of columns of data.

 +-->

<!-- BUGS:
1. Excluded footnotes still get generated as endnotes.
   Footnotes located within required cleanup will be rendered whether the doc is output in draft mode or not.
   This is due to pull-based processing of endnotes, which gets content that would ordinarily be excluded by 
   particular processing settings.  This is actually a general problem faced by ANY XSLT processor that
   has pull-based processors intermingled with push-based exclusion logic.  Ideally, exclusion should remove
   an element from the DOM, then you can depend on pull not to find things it shouldn't find!  Two-passes!

 |Bugs:
 | FO processors are anything but interoperable at the time of this
 | release of the DITA FO demo (August 2002). There are possibly
 | other things to work on as well for this FO implementation, but
 | we know about these:
 |
 |  - this *demo application* has not been fully localization-enabled
 |  - something causes a page overflow error with XEP
 |  - column numbers do not total correctly for FOP (collapses some tables
 |    to the left, although others do appear to work okay)
 |  - "inherit" as an indent value does not work for FOP
 |  - white-space-collapse="false" is provided as an FOP equivalent to
 |    white-space="pre" but doesn't appear to work
 |  - leader-patter="use-content" is ignored by FOP
 |  - FOP starts a blank initial page; AH does not. layout defs may be the cause
 |  - Antenna House requires changing the "dflt-ext" variable to ".jpg"
 |    (and you must have equivalent graphics in the JPEG format)
 |  - the new simpletable element currently spreads evenly across the number of
 |    defined entries; it has a @relativecols attribute to use for future
 |    methods of passing relative widths to its processor (possibly "* * 3*")
 |  - can't get left indent into "lq" so using surround box instead for now
 |
 +  added  start-indent="2pt" into table-cell procs to eliminate deep offsets w/in cells
 |
 +-->



<!DOCTYPE xsl:transform [
<!-- entities for use in the generated output (must produce correctly in FO) -->
  <!ENTITY gt            "&gt;"> 
  <!ENTITY lt            "&lt;"> 
  <!ENTITY rbl           "&#160;">
  <!ENTITY nbsp          "&#160;">
  <!ENTITY quot          "&#34;">
  <!ENTITY quotedblleft  "&#x201C;">
  <!ENTITY quotedblright "&#x201D;">
  <!ENTITY bullet        "&#x2022;"><!--check these two for better assignments -->
  <!ENTITY middot        "&#x2023;">

  <!ENTITY section       "&#xA7;">
  <!ENTITY endash        "&#x2013;">
  <!ENTITY emdash        "&#x2014;">

  <!ENTITY copyr         "&#xA9;">
  <!ENTITY trademark     "&#x2122;">
  <!ENTITY registered    "&#xAE;">
]>

<xsl:transform version="1.0"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">


<!-- OTHER STYLESHEET INCLUDES/IMPORTS -->
<!--xsl:include href="rel-links.xsl"/-->


<!-- OUTPUT METHOD -->

<!-- XSLFO output is simple XML syntax) -->
<xsl:output method="xml" version="1.0"
            indent="no"/>


<!-- USER SPECIFIC DECLARED VALUES  (change as needed) -->

<!-- copyright string for placing into meta/header/footer of each generated topic -->
<xsl:variable name="copyright">Copyright 2003</xsl:variable>


<!-- "GLOBAL" DECLARATIONS -->

<xsl:variable name="img-path"></xsl:variable> <!-- this will be specific to your delivery tree! -->
<xsl:variable name="dflt-ext">.jpg</xsl:variable> <!-- this sets whatever your preferred extension is for your graphics -->
                                                  <!-- For Antenna House, override to ".jpg" -->


<!-- Filler for A-name anchors (empty links)-->
<xsl:variable name="afill"></xsl:variable>

<!-- Filler for empty table entries -->
<xsl:variable name="efill">.</xsl:variable>

<!-- Setup for translation/localization -->
<xsl:variable name="StringFile" select="document('strings.xml')"/>
<xsl:variable name="PrimaryLang" select="substring-before($Lang,'-')"/>


<!-- DEFAULT VALUES FOR EXTERNALLY MODIFIABLE PARAMETERS -->

<!-- /CSS = user's CSS filename parameter ('')-->
<xsl:param name="default-css" select="'common.css'"/>
<xsl:param name="CSS" select="''"/>

<!-- /CSSPATH = default CSS path parameter (null)-->
<xsl:param name="CSSPATH" select="''"/>

<!-- /IP = default image path parameter (null)-->
<xsl:param name="IP" select="''"/>

<!-- /ARTLBL = default "output artwork filenames" processing parameter ('no')-->
<xsl:param name="ARTLBL" select="'no'"/><!-- "no" and "yes" are valid values; non-'yes' is ignored -->

<!-- /DRAFT = default "hide draft & cleanup content" processing parameter ('no' = hide them)-->
<xsl:param name="DRAFT" select="'no'"/><!-- "no" and "yes" are valid values; non-'yes' is ignored -->

<!-- /INDEXSHOW = default "hide index entries" processing parameter ('no' = hide them)-->
<xsl:param name="INDEXSHOW" select="'no'"/><!-- "no" and "yes" are valid values; non-'yes' is ignored -->

<!-- /OUTEXT = default "output extension" processing parameter ('html')-->
<xsl:param name="OUTEXT" select="'html'"/><!-- "htm" and "html" are valid values -->

<!-- /WORKDIR = the working directory that contains the document being transformed.
     Needed as a directory prefix for the @conref and @href "document()" function calls.
     default is './')-->
<xsl:param name="WORKDIR" select="'./'"/>

<!-- /FILENAME = the file name (file name and extension only - no path) of the document being transformed.
     Needed to help with debugging.
     default is 'myfile.xml')-->
<xsl:param name="FILENAME" select="'myfile.xml'"/>

<!-- /DBG = Debug mode - enables XSL debugging xsl-messages.
     Needed to help with debugging.
     default is 'no')-->
<xsl:param name="DBG" select="'no'"/><!-- "no" and "yes" are valid values; non-'yes' is ignored -->

<!-- /Lang = default language parameter  -->
<xsl:param name="Lang" select="'en-us'"/>



<!-- NULL PROCESSORS (for things whose content will be used by reference or excluded) -->

<!-- section title for now is pulled due to need for resolution of possible multiple title methods -->
<xsl:template match="*[contains(@class,' topic/section ')]/*[contains(@class,' topic/title ')]"/>

<!-- fig and table titles can be relocated, therefore must be pulled; can't fall through -->
<xsl:template match="*[contains(@class,' topic/table ')]/*[contains(@class,' topic/title ')]"/>
<xsl:template match="*[contains(@class,' topic/fig ')]/*[contains(@class,' topic/title ')]"/>
<xsl:template match="*[contains(@class,' topic/table ')]/*[contains(@class,' topic/desc ')]"/>
<xsl:template match="*[contains(@class,' topic/fig ')]/*[contains(@class,' topic/desc ')]"/>

<!-- these elements are never processed in a conventional presentation. can be overridden. -->
<xsl:template match="*[contains(@class,' topic/titlealts ')]"/>
<xsl:template match="*[contains(@class,' topic/searchtitle ')]"/>
<xsl:template match="*[contains(@class,' topic/navtitle ')]"/>
<xsl:template match="*[contains(@class,' topic/shortdesc ')]"/>
<xsl:template match="*[contains(@class,' topic/no-topic-nesting ')]"/>

<xsl:template match="*[contains(@class,' topic/prolog ')]"/>
<xsl:template match="*[contains(@class,' topic/metadata ')]"/>
<xsl:template match="*[contains(@class,' topic/author ')]"/>
<xsl:template match="*[contains(@class,' topic/source ')]"/>
<xsl:template match="*[contains(@class,' topic/publisher ')]"/>
<xsl:template match="*[contains(@class,' topic/copyright ')]"/>
<xsl:template match="*[contains(@class,' topic/copyryear ')]"/>
<xsl:template match="*[contains(@class,' topic/copyrholder ')]"/>
<xsl:template match="*[contains(@class,' topic/critdates ')]"/>
<xsl:template match="*[contains(@class,' topic/created ')]"/>
<xsl:template match="*[contains(@class,' topic/revised ')]"/>
<xsl:template match="*[contains(@class,' topic/permissions ')]"/>
<xsl:template match="*[contains(@class,' topic/category ')]"/>
<xsl:template match="*[contains(@class,' topic/audience ')]"/>
<xsl:template match="*[contains(@class,' topic/keywords ')]"/>
<xsl:template match="*[contains(@class,' topic/prodinfo ')]"/>
<xsl:template match="*[contains(@class,' topic/prodname ')]"/>
<xsl:template match="*[contains(@class,' topic/vrmlist ')]"/>
<xsl:template match="*[contains(@class,' topic/vrm ')]"/>
<xsl:template match="*[contains(@class,' topic/brand ')]"/>
<xsl:template match="*[contains(@class,' topic/series ')]"/>
<xsl:template match="*[contains(@class,' topic/platform ')]"/>
<xsl:template match="*[contains(@class,' topic/prognum ')]"/>
<xsl:template match="*[contains(@class,' topic/featnum ')]"/>
<xsl:template match="*[contains(@class,' topic/component ')]"/>
<xsl:template match="*[contains(@class,' topic/othermeta ')]"/>
<xsl:template match="*[contains(@class,' topic/resourceid ')]"/>

<!-- handled by related-links.xsl: related-links, link, linkpool, linklist,
     title and desc in linklist, and xref -->

<xsl:template match="related-links|linktext|linklist|linkpool"/>



<!-- ROOT RULE (just fall through) -->

<xsl:template match="/">
  <xsl:apply-templates mode="toplevel"/>
</xsl:template>




<!-- This first template rule generates the outer-level shell for a delivery context. -->
<!-- In an override stylesheet, the same call to "chapter-setup" must be issued to
     maintain the consistency of overall look'n'feel of the output FO. -->

<xsl:template match="*[contains(@class,' topic/topic ')]" name="toptopic" mode="toplevel">
  <xsl:comment>Common DITA topic to FO application by IBM Corp., 2001, 2002, 2003</xsl:comment>
<!-- this is an "h1div" context for standalone documents -->
  <xsl:call-template name="chapter-setup"/>
</xsl:template>


<!-- =============== start of contextual topic titles ================= -->

<!-- NESTED TOPIC CONTEXTS (child topics get a fo:block wrapper and fall through) -->
<xsl:template match="/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]" priority="2">
  <fo:block><!-- class="nested1"-->
    <xsl:apply-templates select="@id"/>
    <xsl:call-template name="gen-toc-id"/>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]" priority="3">
  <fo:block><!-- class="nested2"-->
    <xsl:apply-templates select="@id"/>
    <xsl:call-template name="gen-toc-id"/>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]" priority="4">
  <fo:block><!-- class="nested3"-->
    <xsl:apply-templates select="@id"/>
    <xsl:call-template name="gen-toc-id"/>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]" priority="5">
  <fo:block><!-- class="nested4"-->

    <xsl:apply-templates select="@id"/>
    <xsl:call-template name="gen-toc-id"/>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]" priority="6">
  <fo:block><!-- class="nested5"-->
    <xsl:apply-templates select="@id"/>
    <xsl:call-template name="gen-toc-id"/>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>



<!-- NESTED TOPIC TITLES (sensitive to nesting depth, but are still processed for contained markup) -->

<!-- h1 -->
<xsl:template match="*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/title ')]" priority="2">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at h1 heading context FOR STANDALONE TOPICS!}</fo:block></xsl:if>
  <xsl:if test="@id"><fo:inline id="{@id}"></fo:inline></xsl:if>
  <fo:block xsl:use-attribute-sets="topictitle1" id="{generate-id()}" line-height="100%">
    <fo:block><fo:leader color="black" leader-pattern="rule" rule-thickness="3pt" leader-length="100%"/></fo:block>
<xsl:variable name="title-txt">
        <xsl:call-template name="get-title"/>
</xsl:variable>
    <fo:block><!-- prefix text? --><xsl:value-of select="$title-txt"/>
      <!--fo:marker marker-class-name="chaptitle">
        <xsl:value-of select="$title-txt"/>
      </fo:marker-->
    </fo:block>
  </fo:block>
  <!--xsl:call-template name="gen-body-qatoc"/--> <!-- only one of these will usually "take" -->
  <!--xsl:call-template name="gen-sect-qatoc"/-->
  <!--xsl:call-template name="gen-ptoc"/-->
</xsl:template>

<!-- h2 -->
<xsl:template match="*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/title ')]" priority="3">
<xsl:if test="$trace='yes'"><fo:block color="purple">{arrived at "name='h2heading'"}</fo:block></xsl:if>
  <fo:block xsl:use-attribute-sets="topictitle2" id="{generate-id()}">
    <fo:block space-after="3pt">
      <fo:leader color="black" leader-pattern="rule" rule-thickness=".5pt" leader-length="100%"/>
    </fo:block>
    <xsl:call-template name="get-title"/>
  </fo:block>
</xsl:template>

<!-- h3 -->
<xsl:template match="*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/title ')]" priority="4">
<xsl:if test="$trace='yes'"><fo:block color="purple">{arrived at "name='h3heading'"}</fo:block></xsl:if>
  <fo:block xsl:use-attribute-sets="topictitle3" id="{generate-id()}">
    <xsl:call-template name="get-title"/>
  </fo:block>
</xsl:template>

<!-- h4 -->
<xsl:template match="*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/title ')]" priority="5">
<xsl:if test="$trace='yes'"><fo:block color="purple">{arrived at "name='h4heading'"}</fo:block></xsl:if>
  <fo:block xsl:use-attribute-sets="topictitle4" id="{generate-id()}">
    <xsl:call-template name="get-title"/>
  </fo:block>
</xsl:template>

<!-- h5 -->
<xsl:template match="*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/title ')]" priority="6">
  <fo:block xsl:use-attribute-sets="topictitle5" id="{generate-id()}">
    <xsl:call-template name="get-title"/><xsl:text>: </xsl:text>
  </fo:block>
</xsl:template>

<!-- h6 -->
<xsl:template match="*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/title ')]" priority="7">
  <fo:block xsl:use-attribute-sets="topictitle6" id="{generate-id()}">
    <xsl:call-template name="get-title"/><xsl:text>: </xsl:text>
  </fo:block>
</xsl:template>


<!-- =============== end of contextual topic titles ================= -->











<!-- ============== modes (out of line processing) ================ -->

<!-- provide fall through processing for title/desc pulled in table or figure contexts -->
<xsl:template match="*[contains(@class,' topic/title ')]" mode="exhibittitle">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/desc ')]" mode="exhibitdesc">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/shortdesc ')]" mode="outofline">
  <fo:block padding-left="2pt" background-color="#e0e0e0">
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>

<!-- ============ end of modes (out of line processing) ============== -->


<xsl:template match="*[contains(@class,' topic/body ')]" name="topbody">
  <fo:block>
    <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <!-- here, you can generate a toc based on what's a child of body -->
    <!--xsl:call-template  name="gen-sect-ptoc"/-->
    <xsl:apply-templates select="parent::*/*[contains(@class,' topic/shortdesc ')]" mode="outofline"/>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/section ')]">
  <fo:block margin-top="15pt" line-height="12pt">
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <xsl:call-template name="gen-toc-id"/>
    <xsl:call-template name="sect-heading">
       <xsl:with-param name="deftitle"></xsl:with-param>
    </xsl:call-template>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/example ')]">
  <fo:block><!-- margin-top="15pt" line-height="12pt"-->
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <xsl:call-template name="gen-toc-id"/>
    <xsl:call-template name="sect-heading">
       <xsl:with-param name="deftitle">Example</xsl:with-param>
    </xsl:call-template>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/p ')]">
  <xsl:apply-templates select="@id"/>
  <xsl:choose>
    <xsl:when test="descendant::*[contains(@class,' topic/pre ')]">
      <xsl:call-template name="divlikepara"/>
    </xsl:when>
    <xsl:when test="descendant::*[contains(@class,' topic/ul ')]">
      <xsl:call-template name="divlikepara"/>
    </xsl:when>
    <xsl:when test="descendant::*[contains(@class,' topic/ol ')]">
      <xsl:call-template name="divlikepara"/>
    </xsl:when>
    <xsl:when test="descendant::*[contains(@class,' topic/lq ')]">
      <xsl:call-template name="divlikepara"/>
    </xsl:when>
    <xsl:when test="descendant::*[contains(@class,' topic/dl ')]">
      <xsl:call-template name="divlikepara"/>
    </xsl:when>
    <xsl:when test="descendant::*[contains(@class,' topic/note ')]">
      <xsl:call-template name="divlikepara"/>
    </xsl:when>
    <xsl:when test="descendant::*[contains(@class,' topic/lines ')]">
      <xsl:call-template name="divlikepara"/>
    </xsl:when>
    <xsl:when test="descendant::*[contains(@class,' topic/fig ')]">
      <xsl:call-template name="divlikepara"/>
    </xsl:when>
    <xsl:when test="descendant::*[contains(@class,' topic/table ')]">
      <xsl:call-template name="divlikepara"/>
    </xsl:when>
    <xsl:when test="descendant::*[contains(@class,' topic/simpletable ')]">
      <xsl:call-template name="divlikepara"/>
    </xsl:when>
    <xsl:otherwise>
      <fo:block xsl:use-attribute-sets="p">
        <xsl:if test="string(@id)"><xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute></xsl:if>

        <xsl:apply-templates/>
      </fo:block>
    </xsl:otherwise>
  </xsl:choose>    
</xsl:template>

<xsl:template name="divlikepara">
  <fo:block xsl:use-attribute-sets="divlike.p">

    <xsl:apply-templates/>
  </fo:block>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/note ')]">
  <fo:block xsl:use-attribute-sets="p" space-after.optimum="6pt" margin-left="+2pc" start-indent="-2pc">
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <fo:wrapper font-weight="bold">
    <xsl:choose>
      <xsl:when test="@type='note'">
        <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Note'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@type='tip'">
      <xsl:call-template name="proc-ing"/>
        <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Tip'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@type='fastpath'">
        <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Fastpath'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@type='restriction'">
        <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Restriction'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@type='important'">>
        <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Important'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@type='remember'">
        <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Remember'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@type='attention'">
        <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Attention'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@type='caution'">
        <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Caution'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@type='danger'">
        <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Danger'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@type='other'">
        <xsl:choose>
          <xsl:when test="@othertype">
          <!-- othertype is a key that should look up external, translateable text. -->
           <xsl:value-of select="@othertype"/>
          </xsl:when>
          <xsl:otherwise>
            <!-- nop -->
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
          <!-- nop -->
      </xsl:otherwise>
    </xsl:choose><xsl:text>: </xsl:text>
    </fo:wrapper>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/lq ')]">
  <fo:block xsl:use-attribute-sets="lq">
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <xsl:apply-templates/>
    <xsl:choose>
      <xsl:when test="@href">
        <fo:block text-align="right"><xsl:value-of select="@href"/>,
<xsl:value-of select="@reftitle"/></fo:block>
      </xsl:when>
      <xsl:when test="@reftitle">
        <fo:block text-align="right"><xsl:value-of select="@reftitle"/></fo:block>
      </xsl:when>
      <xsl:otherwise><!--nop--></xsl:otherwise>
    </xsl:choose>
  </fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/q ')]">
 <fo:inline>&quotedblleft;<xsl:apply-templates />&quotedblright;</fo:inline>
</xsl:template>


<xsl:template name="li.name">
     <xsl:if test="string(@id)"><fo:inline id="{@id}"> </fo:inline></xsl:if>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/ul ')]|*[contains(@class,' topic/ol ')]">
<xsl:variable name="list-type" select="name()" /> 
<xsl:variable name="list-level" select="count(ancestor-or-self::*[name()=$list-type])" /> 
  <fo:list-block provisional-distance-between-starts="3pt"
                 provisional-label-separation="3pt">
    <xsl:apply-templates/>
  </fo:list-block>
</xsl:template>

<!-- TBD: 2nd level through 9th level: alternate through default, type="a", type="i" -->


<!-- add "compact" awareness here -->
<xsl:template match="*[contains(@class,' topic/ol ')]/*[contains(@class,' topic/lix ')]">
<xsl:variable name="list-level" select="count(ancestor-or-self::*[contains(@class,' topic/ul ')] | 
                                         ancestor-or-self::*[contains(@class,' topic/dl ')] |
                                         ancestor-or-self::*[contains(@class,' topic/ol ')] )" />
  <fo:list-item space-before="6pt"> <!--.optimum="4pt", or deal with "compact=''" value -->
  <fo:list-item-label  end-indent="label-end()">
    <xsl:attribute name="start-indent">
      <xsl:choose>
        <xsl:when test="$list-level=1">
          <xsl:value-of select="$basic-start-indent"/></xsl:when>
        <xsl:otherwise> + 12pt</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <fo:block ><xsl:number format="1."/></fo:block>
  </fo:list-item-label>
  <fo:list-item-body>
    <xsl:attribute name="start-indent">
      <xsl:choose>
        <xsl:when test="$list-level=1">
          <xsl:value-of select="$basic-start-indent"/> + 16pt</xsl:when>
        <xsl:otherwise> + 24pt</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <fo:block>
      <xsl:if test="string(@id)"><fo:inline id="{@id}"> </fo:inline></xsl:if>
<!--[<xsl:value-of select="$basic-start-indent"/>] (produces[72])-->
      <xsl:apply-templates/>
    </fo:block>
  </fo:list-item-body>
  </fo:list-item>
</xsl:template>

<!-- 
 Ordered list item                                               
--> 
<xsl:template match="*[contains(@class,' topic/ol ')]/*[contains(@class,' topic/li ')]">
  <xsl:variable name="list-level" select="count(ancestor-or-self::*[contains(@class,' topic/ul ')] | 
                                         ancestor-or-self::*[contains(@class,' topic/dl ')] |
                                         ancestor-or-self::*[contains(@class,' topic/ol ')] )" />
  <fo:list-item> 
    <fo:list-item-label end-indent="label-end()"> 
      <fo:block text-align="start"> 
        <xsl:choose> 
          <xsl:when test="($list-level mod 2) = 1"> 
            <!--          arabic         --> 
            <xsl:number format="1." /> 
          </xsl:when> 
          <xsl:otherwise> 
            <!--              alphabetic             --> 
            <xsl:number format="a." /> 
          </xsl:otherwise> 
        </xsl:choose> 
      </fo:block> 
    </fo:list-item-label> 
    <fo:list-item-body start-indent="body-start()"> 
           <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/> + 16pt</xsl:attribute>
     <fo:block> 
        <xsl:apply-templates /> 
      </fo:block> 
    </fo:list-item-body> 
  </fo:list-item> 
</xsl:template>



<xsl:template match="ul/*[contains(@class,' topic/li ')]">
  <xsl:variable name="list-level" select="count(ancestor-or-self::*[contains(@class,' topic/ul ')] | 
                                         ancestor-or-self::*[contains(@class,' topic/dl ')] |
                                         ancestor-or-self::*[contains(@class,' topic/ol ')] )" />
  <fo:list-item space-before="1pt"> <!--.optimum="4pt", or deal with "compact=''" value -->
  <fo:list-item-label end-indent="label-end()">
    <xsl:attribute name="start-indent">
      <xsl:choose>
        <xsl:when test="$list-level=1">
          <xsl:value-of select="$basic-start-indent"/>
        </xsl:when>
        <xsl:otherwise>
          inherit + 24pt
        </xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <fo:block linefeed-treatment="ignore">
      <fo:inline font-family="Symbol">&bullet;</fo:inline>
    </fo:block>
  </fo:list-item-label>
  <fo:list-item-body>
    <xsl:attribute name="start-indent">
      <xsl:choose>
        <xsl:when test="$list-level=1">
          inherit + 12pt
        </xsl:when>
        <xsl:otherwise>
          inherit + 24pt
        </xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <fo:block linefeed-treatment="ignore">
      <xsl:if test="string(@id)"><fo:inline id="{@id}"> </fo:inline></xsl:if>
      <xsl:apply-templates/>
    </fo:block>
  </fo:list-item-body>
  </fo:list-item>
</xsl:template>



<xsl:template match="*[contains(@class,' topic/itemgroup ')]">
  <fo:block xsl:use-attribute-sets="p">
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@compact"/>
    <xsl:apply-templates select="@id"/>
    <xsl:apply-templates />
  </fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/dl ')]">
  <xsl:apply-templates/>
</xsl:template>


<xsl:template match="*[contains(@class, ' topic/dlhead ')]">
  <fo:block font-weight="bold" text-decoration="underline">
    <xsl:apply-templates />
  </fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/dthd ')]">          
  <fo:block xsl:use-attribute-sets="dt" font-weight="bold">
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <xsl:call-template name="apply-for-phrases"/>
  </fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/ddhd ')]">
  <fo:block xsl:use-attribute-sets="dd" font-weight="bold">
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <xsl:call-template name="apply-for-phrases"/>
  </fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/dlentry ')]">

  <xsl:if test="@id">
  <xsl:call-template name="output-message">
    <xsl:with-param name="msg">The ID attribute on DLENTRY will be ignored. Move that ID to a DT tag.</xsl:with-param>
    <xsl:with-param name="msgnum">002</xsl:with-param>
    <xsl:with-param name="msgsev">W</xsl:with-param>
  </xsl:call-template>
  </xsl:if>
  <xsl:apply-templates/>
</xsl:template>



<xsl:template match="*[contains(@class,' topic/dt ')]">

 <fo:block xsl:use-attribute-sets="dt">
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <xsl:choose>
      <xsl:when test="*"> <!-- tagged content - do not default to bold -->
        <xsl:apply-templates/>
      </xsl:when>
      <xsl:otherwise>
        <fo:inline font-weight="bold"><xsl:call-template name="apply-for-phrases"/></fo:inline> <!-- text only - bold it -->
      </xsl:otherwise>
    </xsl:choose>
 </fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/dd ')]">
  <fo:block xsl:use-attribute-sets="dd">
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
   <xsl:apply-templates />
  </fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/fig ')]">
<fo:block xsl:use-attribute-sets="fig">
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
  <!-- apply label within the block so that it floats with the fig itself -->
  <xsl:if test="$fmt-fig-lbl-loc='over'"><xsl:call-template name="place-fig-lbl"/></xsl:if>
  <!-- support "frame='all'"-->
  <!-- support "frame='top'"-->
  <xsl:if test="contains(@frame,'top')">
    <fo:block space-after="3pt">
      <fo:leader color="black" leader-pattern="rule" rule-thickness=".5pt" leader-length="100%"/>
    </fo:block>
  </xsl:if>

  <fo:block><!-- created JUST for styling the exhibit area -->
  <xsl:if test="contains(@frame,'all')"><!-- adorn currently open block -->
    <xsl:attribute name="border-style">solid</xsl:attribute>
    <xsl:attribute name="border-color">black</xsl:attribute>
    <xsl:attribute name="border-width">thin</xsl:attribute>
    <!--xsl:attribute name="padding">6pt</xsl:attribute-->
  </xsl:if> 
  <!-- this is where the main fig rendering happens -->
  <xsl:apply-templates/>
  </fo:block><!-- created JUST for styling the exhibit area -->

  <!-- support "frame='bot'"-->
  <xsl:if test="contains(@frame,'bot')">
    <fo:block space-after="3pt">
      <fo:leader color="black" leader-pattern="rule" rule-thickness=".5pt" leader-length="100%"/>
    </fo:block>

  </xsl:if>
  <!-- apply label within the block so that it floats with the fig itself -->
  <xsl:if test="$fmt-fig-lbl-loc='under'"><xsl:call-template name="place-fig-lbl"/></xsl:if>
</fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/figgroup ')]">
  <fo:inline>
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <xsl:apply-templates/>
  </fo:inline>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/pre ')]">
  <xsl:call-template name="gen-att-label"/>
  <fo:block xsl:use-attribute-sets="pre">
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <xsl:call-template name="setscale"/>
<!-- rules have to be applied within the scope of the PRE box; else they start from page margin! -->
    <xsl:if test="contains(@frame,'top')"><fo:block><fo:leader leader-pattern="rule" leader-length="5.65in" /></fo:block></xsl:if>
    <xsl:apply-templates/>
    <xsl:if test="contains(@frame,'bot')"><fo:block><fo:leader leader-pattern="rule" leader-length="5.65in" /></fo:block></xsl:if>
  </fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/lines ')]">
  <xsl:call-template name="gen-att-label"/>
  <fo:block xsl:use-attribute-sets="lines">
     <xsl:call-template name="setclass"/>
     <xsl:apply-templates select="@id"/>
     <xsl:call-template name="setscale"/>
     <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
     <xsl:apply-templates/>
  </fo:block>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/term ')]">
  <fo:inline>
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <xsl:apply-templates/>
  </fo:inline>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/ph ')]">
  <fo:inline color="purple">
    <xsl:call-template name="setclass"/>

    <xsl:apply-templates select="@id"/>
    <xsl:apply-templates/>
  </fo:inline>
</xsl:template>



<xsl:template match="*[contains(@class,' topic/tm ')]">
  <fo:inline>
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <xsl:apply-templates/>
    <fo:inline baseline-shift="super" font-size="75%">
    <xsl:choose>
      <xsl:when test="@tmtype='tm'">(TM)</xsl:when>
      <xsl:when test="@tmtype='reg'">(R)</xsl:when>
      <xsl:when test="@tmtype='service'">(SM)</xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="output-message">
          <xsl:with-param name="msg">Invalid trademark attribute.</xsl:with-param>
          <xsl:with-param name="msgnum">003</xsl:with-param>
          <xsl:with-param name="msgsev">W</xsl:with-param>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
    </fo:inline>
  </fo:inline>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/boolean ')]">
  <fo:inline color="green">
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <xsl:value-of select="name()"/><xsl:text>: </xsl:text><xsl:value-of select="@state"/>
  </fo:inline>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/state ')]">
  <fo:inline color="red">
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <xsl:value-of select="name()"/><xsl:text>: </xsl:text><xsl:value-of select="@name"/><xsl:text>=</xsl:text><xsl:value-of select="@value"/>
  </fo:inline>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/image ')]">
  <!-- build any pre break indicated by style -->
  <xsl:choose>
    <xsl:when test="parent::fig[contains(@frame,'top ')]">
      <!-- NOP if there is already a break implied by a parent property -->
    </xsl:when>
    <xsl:otherwise>
      <xsl:if test="not(@placement='inline')">
        <!-- generate an FO break here -->
      </xsl:if>
    </xsl:otherwise>
  </xsl:choose>
  <fo:block>
     <xsl:choose>
     <xsl:when test="@align='left'"><xsl:attribute name="class">imageleft</xsl:attribute></xsl:when>
     <xsl:when test="@align='right'"><xsl:attribute name="class">imageright</xsl:attribute></xsl:when>
     <xsl:when test="@align='center'"><xsl:attribute name="class">imagecenter</xsl:attribute></xsl:when>
     <xsl:when test="@align='justify'"><xsl:attribute name="class">imagejustify</xsl:attribute></xsl:when>
     <xsl:otherwise/>
    </xsl:choose>
    <!--xsl:call-template name="topic-image"/-->
    <xsl:call-template name="unused-insert-graphic"/>
  </fo:block>
  <!-- build any post break indicated by style -->
  <xsl:if test="not(@placement='inline')"><br clear="all"/></xsl:if>
  <!-- build optional echo of the image name for review -->
  <xsl:if test="$ARTLBL='yes'">
     <fo:block font-weight="bold">[<xsl:value-of select="$IP"/> <xsl:value-of select="@href"/>]</fo:block>
  </xsl:if>
</xsl:template>


<xsl:template name="unused-insert-graphic">
   <fo:external-graphic> <!-- apply scaling attributes here? -->
      <!--hard force the extension to jpg -->
     <xsl:variable name="thisfn">
       <xsl:choose>
         <xsl:when test="contains(@href,'.gif')">
           <xsl:value-of select="concat($img-path, substring-before(@href,'.gif'), $dflt-ext)"/>
         </xsl:when>
         <xsl:when test="contains(@href,'.jpg')">
           <xsl:value-of select="concat($img-path, substring-before(@href,'.jpg'), $dflt-ext)"/>
         </xsl:when>
         <xsl:otherwise>
           <xsl:value-of select="@href"/>
         </xsl:otherwise>
       </xsl:choose>
     </xsl:variable>
     <xsl:attribute name="src">url(<xsl:value-of select="$thisfn"/>)</xsl:attribute>
     <xsl:choose>
       <xsl:when test="@scale"><xsl:attribute name="content-width"><xsl:value-of select="concat(@scale,'%')"/></xsl:attribute></xsl:when>
       <xsl:otherwise><!--xsl:attribute name="content-width">scale-to-fit</xsl:attribute--></xsl:otherwise> <!-- no effect -->
     </xsl:choose>
   </fo:external-graphic>
</xsl:template>

<!-- this is unused for now, but is HTML specific; must be converted! -->
<xsl:template name="topic-image">
  <!-- now invoke the actual content and its alt text -->
  <xsl:element name="img">
    <xsl:call-template name="setclass"/>
    <xsl:attribute name="src">
     <xsl:value-of select="$IP"/>
       <xsl:choose>
         <xsl:when test="@objname">
           <!--xsl:call-template name="get-objdescinfo"/-->
         </xsl:when>
         <xsl:when test="@href">
           <xsl:value-of select="@href"/>
         </xsl:when>
         <xsl:otherwise>
           <!-- no action -->
         </xsl:otherwise>
       </xsl:choose>
     </xsl:attribute>
    <xsl:if test="@height"><xsl:attribute name="height"><xsl:value-of select="@height"/></xsl:attribute></xsl:if>
    <xsl:if test="@width"><xsl:attribute name="width"><xsl:value-of select="@width"/></xsl:attribute></xsl:if>
    <xsl:choose>
      <xsl:when test="@alt">
        <xsl:attribute name="alt"><xsl:value-of select="@alt"/></xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="alt"><xsl:value-of select="*[contains(@class,' topic/textalt')]"/></xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:element>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/object ')]">
  <!-- copy through for HTML; unused for FO -->
</xsl:template>


<xsl:template match="*[contains(@class,' topic/param ')]">
  <!-- copy through for HTML; unused for FO -->
</xsl:template>


<xsl:template match="*[contains(@class,' topic/simpletable ')]">
  <fo:table xsl:use-attribute-sets="table.data frameall">
    <xsl:call-template name="semtbl-colwidth"/>
    <fo:table-body>
      <xsl:call-template name="gen-dflt-data-hdr"/>
      <xsl:apply-templates/>
    </fo:table-body>
  </fo:table>
</xsl:template>


<!-- if cells have default spectitle values, copy these up to a first new hdr row -->
<xsl:template name="gen-dflt-data-hdr">
  <xsl:if test="*/*[contains(@class,' topic/stentry ')]/@specentry">
    <fo:table-row>
       <xsl:for-each select="*[contains(@class,' topic/strow ')][1]/*">
          <fo:table-cell start-indent="2pt" background-color="silver" padding="2pt" text-align=
            "center" font-weight="bold" xsl:use-attribute-sets="frameall">
            <xsl:attribute name="column-number"><xsl:number count="*"/></xsl:attribute>
            <fo:block>
            <xsl:value-of select="@specentry"/>
            </fo:block>
          </fo:table-cell>
        </xsl:for-each>
    </fo:table-row>
  </xsl:if>
</xsl:template>



<xsl:template name="semtbl-colwidth">
  <!-- Find the total number of relative units for the table. If @relcolwidth="1* 2* 2*",
       the variable is set to 5. -->
  <xsl:variable name="totalwidth">
    <xsl:if test="not(@relcolwidth='')">
      <xsl:call-template name="find-total-table-width"/>
    </xsl:if>
  </xsl:variable>
  <!-- Find how much of the table each relative unit represents. If @relcolwidth is 1* 2* 2*,
       there are 5 units. So, each unit takes up 100/5, or 20% of the table. Default to 0,
       which the entries will ignore. -->
  <!-- bad relcolwidth data will still generate "NaN" errors, however -->
  <xsl:variable name="width-multiplier">
    <xsl:choose>
      <xsl:when test="not(@relcolwidth='')">
        <xsl:value-of select="100 div $totalwidth"/>
      </xsl:when>
      <xsl:otherwise>0</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>


  <xsl:for-each select="strow[1]/*"><!-- use cells as counters; avoid recursion! -->
    <!-- Determine which column this entry is in. -->
    <xsl:variable name="thiscolnum"><xsl:value-of select="position()"/></xsl:variable>
    <fo:table-column>
      <xsl:attribute name="column-width">
      <xsl:choose>
        <xsl:when test="$width-multiplier != 0">
          <xsl:variable name="thispct">
            <xsl:call-template name="get-current-entry-percentage">
              <xsl:with-param name="multiplier"><xsl:value-of select="$width-multiplier"/></xsl:with-param>
              <xsl:with-param name="entry-num"><xsl:value-of select="$thiscolnum"/></xsl:with-param>
            </xsl:call-template>
          </xsl:variable> 
          <xsl:value-of select="number(198 * $thispct div 100 div 36)"/>in</xsl:when>
        <xsl:otherwise>0</xsl:otherwise>
      </xsl:choose>
      </xsl:attribute>
    </fo:table-column>
  </xsl:for-each>
</xsl:template>








<!-- Use @relcolwidth to find the total width of the table. That is, if the attribute is set
     to 1* 2* 2* 1*, then the table is 6 units wide. -->
<xsl:template name="find-total-table-width">
  <!-- Start with relcolwidth, and each recursive call will remove the first value -->
  <xsl:param name="relcolwidth"><xsl:value-of select="@relcolwidth"/></xsl:param>
  <!-- Determine the first value, which is the value before the first asterisk -->
  <xsl:variable name="firstval">
    <xsl:if test="contains($relcolwidth,'*')">
      <xsl:value-of select="substring-before($relcolwidth,'*')"/>
    </xsl:if>
  </xsl:variable>
  <!-- Begin processing if we were able to find a first value -->
  <xsl:if test="string-length($firstval)>0">
    <!-- Chop off the first value, and set morevals to the remainder -->
    <xsl:variable name="morevals"><xsl:value-of select="substring-after($relcolwidth,' ')"/></xsl:variable>
    <xsl:choose>
      <!-- If there are additional values, call this template on the remainder.
           Add the result of that call to the first value. -->
      <xsl:when test="string-length($morevals)>0">
        <xsl:variable name="nextval">   <!-- The total of the remaining values -->
          <xsl:call-template name="find-total-table-width">
            <xsl:with-param name="relcolwidth"><xsl:value-of select="$morevals"/></xsl:with-param>
          </xsl:call-template>
        </xsl:variable>
        <xsl:value-of select="number($firstval)+number($nextval)"/>
      </xsl:when>
      <!-- If there are no more values, return the first (and only) value -->
      <xsl:otherwise><xsl:value-of select="$firstval"/></xsl:otherwise>
    </xsl:choose>
  </xsl:if>
</xsl:template>

<!-- Find the width of the current cell. Multiplier is how much each unit of width is multiplied to total 100.
     Entry-num is the current entry. Current-col is what column we are at when scanning @relcolwidth.
     Relcolvalues is the unscanned part of @relcolwidth. -->
<xsl:template name="get-current-entry-percentage">
  <xsl:param name="multiplier">1</xsl:param>  <!-- Each relative unit is worth this many percentage points -->
  <xsl:param name="entry-num"/>               <!-- The entry number of the cell we are evaluating now -->
  <xsl:param name="current-col">1</xsl:param> <!-- Position within the recursive call to evaluate @relcolwidth -->
  <!-- relcolvalues begins with @relcolwidth. Each call to the template removes the first value. -->
  <xsl:param name="relcolvalues"><xsl:value-of select="parent::*/parent::*/@relcolwidth"/></xsl:param>

  <xsl:choose>
    <!-- If the recursion has moved up to the proper cell, multiply $multiplier by the number of
         relative units for this column. -->
    <xsl:when test="$entry-num = $current-col">
      <xsl:variable name="relcol"><xsl:value-of select="substring-before($relcolvalues,'*')"/></xsl:variable>
      <xsl:value-of select="$relcol * $multiplier"/>
    </xsl:when>
    <!-- Otherwise, call this template again, removing the first value form @relcolwidth. Also add one
         to $current-col. -->
    <xsl:otherwise>
      <xsl:call-template name="get-current-entry-percentage">
        <xsl:with-param name="multiplier"><xsl:value-of select="$multiplier"/></xsl:with-param>
        <xsl:with-param name="entry-num"><xsl:value-of select="$entry-num"/></xsl:with-param>
        <xsl:with-param name="current-col"><xsl:value-of select="$current-col + 1"/></xsl:with-param>
        <xsl:with-param name="relcolvalues"><xsl:value-of select="substring-after($relcolvalues,' ')"/></xsl:with-param>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>







<xsl:template match="*[contains(@class,' topic/sthead ')]">
  <fo:table-row>
    <xsl:apply-templates/>
  </fo:table-row>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/strow ')]">
  <fo:table-row>
    <xsl:apply-templates/>
  </fo:table-row>
</xsl:template>


<!-- in the following two rules, the xsl:number needs to give the
  sequence number of either the current semdh or sementry in the current
  semrow OR semhdr. -->

<xsl:template match="*[contains(@class,' topic/sthead ')]/*[contains(@class,' topic/stentry ')]" priority="2">
  <fo:table-cell start-indent="2pt" background-color="silver" padding="2pt" text-align=
"center" font-weight="bold" xsl:use-attribute-sets="frameall">
    <xsl:attribute name="column-number"><xsl:number count="*"/></xsl:attribute>
    <fo:block>
    <xsl:call-template name="get-title"/>
    </fo:block>
  </fo:table-cell>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/stentry ')]">
  <fo:table-cell start-indent="2pt" background-color="white" padding="2pt" xsl:use-attribute-sets="frameall">
    <xsl:attribute name="column-number"><xsl:number count="*"/></xsl:attribute>
    <fo:block>
    <xsl:apply-templates/>
    </fo:block>
  </fo:table-cell>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/draft-comment ')]">
  <xsl:if test="$DRAFT='yes'">
    <fo:block background-color="#FF99FF" color="#CC3333" border="1pt black solid">
      <xsl:apply-templates select="@id"/>
      <fo:inline font-weight="bold">Comment by <xsl:value-of select="@reviewer"/></fo:inline> 
       <xsl:apply-templates/>
    </fo:block>
  </xsl:if>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/required-cleanup ')]">
  <xsl:if test="$DRAFT='yes'">
    <xsl:call-template name="output-message">
      <xsl:with-param name="msg">Required cleanup area found.</xsl:with-param>
      <xsl:with-param name="msgnum">001</xsl:with-param>
      <xsl:with-param name="msgsev">W</xsl:with-param>
    </xsl:call-template>



    <fo:inline background="yellow" color="#CC3333">  <!-- indents won't apply here; not a block context -->
      <xsl:apply-templates select="@id"/>
      <fo:inline font-weight="bold">Required Cleanup <xsl:if test="string(@remap)">(<xsl:value-of select="@remap"/>) </xsl:if><xsl:text>: </xsl:text></fo:inline> 
      <xsl:apply-templates />
    </fo:inline>
  </xsl:if>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/fn ')]">
<fo:footnote>
  <fo:inline baseline-shift="super" font-size="75%" font-family="Helvetica">
    <xsl:number count="//fn"    format="1" />
  </fo:inline>
  <fo:footnote-body>
    <!--fo:block space-after="0pt">
      <fo:leader color="black" leader-pattern="rule" rule-thickness=".5pt" leader-length="1in"/>
    </fo:block-->
    <fo:block xsl:use-attribute-sets="footnote">
      <fo:inline baseline-shift="super" font-size="75%">
         <xsl:number level="any" count="//fn"  format="1 "/>
      </fo:inline>
      <xsl:apply-templates />
    </fo:block>
  </fo:footnote-body>
</fo:footnote>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/cite ')]">
  <fo:inline font-style="italic">
    <xsl:call-template name="setclass"/>
    <xsl:apply-templates select="@id"/>
    <xsl:apply-templates />
  </fo:inline>
</xsl:template>


<xsl:template match="*[contains(@class,' topic/indextermref ')]"/>


<xsl:template match="*[contains(@class,' topic/indexterm ')]">
  <xsl:if test="$INDEXSHOW='yes'">
    <fo:inline margin="1pt" background-color="#ffddff"><!-- border="1pt black solid;"-->
      <xsl:call-template name="setclass"/>
      <xsl:apply-templates/>
    </fo:inline>
  </xsl:if>
</xsl:template>



<xsl:template match="*[contains(@class,' topic/keyword ')]">
  <xsl:apply-templates select="@id"/>
  <xsl:apply-templates/>
</xsl:template>


<!-- CALS (OASIS) TABLE -->
<!-- adapted from sample code in the XSL FO spec; extended considerably! -->



<xsl:template match="table/title" priority="2"/>

<xsl:template match="*[contains(@class,' topic/table ')]">
  <xsl:choose>
    <xsl:when test="xtitle"><!-- renamed for now to NOT trigger on title; testing the "otherwise" code -->
      <fo:table-and-caption xsl:use-attribute-sets="table.data frameall">
        <fo:table-caption>
          <fo:block xsl:use-attribute-sets="table.data.caption">
            <xsl:value-of select="title"/>
          </fo:block>
        </fo:table-caption>
        <fo:table xsl:use-attribute-sets="table.data frameall">
          <xsl:if test="string(@id)"><xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute></xsl:if>
<!-- TBD: apply tblatts and place-tbl-width here -->
          <xsl:apply-templates/>
        </fo:table>
      </fo:table-and-caption>
    </xsl:when>
    <xsl:otherwise>
      <fo:table xsl:use-attribute-sets="table.data frameall">
        <xsl:if test="string(@id)"><xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute></xsl:if>
        <xsl:call-template name="default-colwidth"/>
<!-- TBD: apply tblatts and place-tbl-width here -->
        <xsl:apply-templates/>
      </fo:table>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="default-colwidth">
    <!--fo:table-column column-width="150pt"/-->
    <xsl:for-each select="row[1]/entry">
      <fo:table-column>
        <!-- compute even columns (creates valid processing, but all columns are even width) -->
        <xsl:attribute name="column-width"><xsl:value-of select="floor(@width div 72)"/>in</xsl:attribute>
        <!-- TBD: get this value directly from markup, if possible, for adjusted overrides -->
      </fo:table-column>
    </xsl:for-each>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/spanspec ')]">
<!--  <xsl:call-template name="att-align"/> -->
</xsl:template>

<xsl:template match="*[contains(@class,' topic/colspec ')]">
  <fo:table-column>
    <xsl:call-template name="att-align"/>
    <xsl:attribute name="column-number">
      <xsl:number count="colspec"/>
    </xsl:attribute>
    <xsl:attribute name="column-width">
      <xsl:call-template name="xcalc.column.width">
        <xsl:with-param name="colwidth">
          <xsl:value-of select="@colwidth"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:attribute>
  </fo:table-column>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/tgroup ')]">
<!--
  <xsl:call-template name="att-align"/>
-->
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/thead ')]">
  <fo:table-header>
    <xsl:call-template name="att-valign"/>
    <xsl:apply-templates />
  </fo:table-header>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/tfoot ')]">
  <fo:table-footer>
    <xsl:call-template name="att-valign"/>
    <xsl:apply-templates/>
  </fo:table-footer>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/tbody ')]">
  <fo:table-body>
    <xsl:call-template name="att-valign"/>
    <xsl:apply-templates/>
  </fo:table-body>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/tnote ')]">
  <fo:table-row>
  <fo:table-cell start-indent="2pt" xsl:use-attribute-sets="table.data.td">
    <fo:block><fo:inline font-weight="bold">
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Note'"/>
      </xsl:call-template> <xsl:text>: </xsl:text></fo:inline><xsl:apply-templates/></fo:block>
  </fo:table-cell>
  </fo:table-row>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/row ')]">
  <fo:table-row>
    <xsl:call-template name="att-valign"/>
    <xsl:apply-templates/>
  </fo:table-row>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/thead ')]/*[contains(@class,' topic/row ')]/*[contains(@class,' topic/entry ')]">
<xsl:variable name="colnumval">
  <xsl:choose>
    <xsl:when test="@colname">
      <xsl:value-of select="ancestor::tgroup/colspec[@colname=current()/@colname]/@colnum"/>
    </xsl:when>
    <xsl:when test="@colnum">
      <xsl:value-of select="@colnum"/>
    </xsl:when>
    <xsl:otherwise><xsl:number count="entry"/></xsl:otherwise>
  </xsl:choose>
</xsl:variable>
  <fo:table-cell start-indent="2pt" column-number="{$colnumval}" xsl:use-attribute-sets="table.data.th">
    <xsl:call-template name="entryatts"/>
    <fo:block>
      <xsl:call-template name="fillit"/>
    </fo:block>
  </fo:table-cell>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/tfoot ')]/*[contains(@class,' topic/row ')]/*[contains(@class,' topic/entry ')]">
<xsl:variable name="colnumval">
  <xsl:choose>
    <xsl:when test="@colname">
      <xsl:value-of select="ancestor::tgroup/colspec[@colname=current()/@colname]/@colnum"/>
    </xsl:when>
    <xsl:when test="@colnum">
      <xsl:value-of select="@colnum"/>
    </xsl:when>
    <xsl:otherwise><xsl:number count="entry"/></xsl:otherwise>
  </xsl:choose>
</xsl:variable>
  <fo:table-cell start-indent="2pt" column-number="{$colnumval}" xsl:use-attribute-sets="table.data.tf frameall">
    <xsl:call-template name="entryatts"/>
    <fo:block>
      <xsl:call-template name="fillit"/>
    </fo:block>
  </fo:table-cell>
</xsl:template>


<xsl:template name="get-colnumval">
<xsl:variable name="colnumval">
  <xsl:choose>
    <xsl:when test="@colname">
      <xsl:value-of select="ancestor::tgroup/colspec[@colname=current()/@colname]/@colnum"/>
    </xsl:when>
    <xsl:when test="@colnum">
      <xsl:value-of select="@colnum"/>
    </xsl:when>
    <xsl:otherwise><xsl:number count="entry"/></xsl:otherwise>
  </xsl:choose>
</xsl:variable>
</xsl:template>

<xsl:template match="*[contains(@class,' topic/tbody ')]/*[contains(@class,' topic/row ')]/*[contains(@class,' topic/entry ')]">
<xsl:variable name="colnumval">
  <xsl:choose>
    <xsl:when test="@colname">
      <xsl:value-of select="ancestor::tgroup/colspec[@colname=current()/@colname]/@colnum"/>
    </xsl:when>
    <xsl:when test="@colnum">
      <xsl:value-of select="@colnum"/>
    </xsl:when>
    <xsl:otherwise><xsl:number count="entry"/></xsl:otherwise>
  </xsl:choose>
</xsl:variable>
  <fo:table-cell start-indent="2pt" column-number="{$colnumval}" xsl:use-attribute-sets="table.data.td frameall">
    <xsl:call-template name="entryatts"/>
    <fo:block>
      <xsl:call-template name="fillit"/>
    </fo:block>
  </fo:table-cell>
</xsl:template>


<!-- named templates -->
<xsl:template name="fillit">
<xsl:choose>
  <!-- test to see if the cell contains any sensible text or other element... -->
  <xsl:when test="not(text()[normalize-space(.)] | *)"><fo:inline>&rbl;'</fo:inline></xsl:when>
  <xsl:otherwise><xsl:apply-templates/></xsl:otherwise>
</xsl:choose>
</xsl:template>


<!-- table "macros" -->


<xsl:template name="entryatts">
  <xsl:call-template name="att-valign"/>
   <xsl:call-template name="att-align"/>

  <xsl:if test="string(@colsep)">
    <xsl:if test="@colsep='1'"><xsl:attribute name="cellpadding">10</xsl:attribute></xsl:if>
  </xsl:if>

<!-- IPL start -->
  <xsl:if test="string(@namest)">
    <xsl:variable name="colst" select="substring-after(@namest,'col')"/>
    <xsl:variable name="colend" select="substring-after(@nameend,'col')"/>
    <xsl:attribute name="number-columns-spanned"><xsl:value-of select="$colend - $colst + 1"/></xsl:attribute>
  </xsl:if>
<!-- IPL end -->
</xsl:template>


<xsl:template name="att-valign">
  <xsl:if test="string(@valign)">
    <xsl:choose>
      <xsl:when test="@valign='middle'">
        <xsl:attribute name="display-align">center</xsl:attribute>
      </xsl:when>
      <xsl:when test="@valign='top'">
        <xsl:attribute name="display-align">before</xsl:attribute>
      </xsl:when>
      <xsl:when test="@valign='bottom'">
        <xsl:attribute name="display-align">after</xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="display-align">
          <xsl:value-of select="@valign"/>
        </xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:if>
</xsl:template>


<xsl:template name="att-align">
  <xsl:if test="string(@align)">
    <xsl:attribute name="text-align">
      <xsl:value-of select="normalize-space(@align)"/>


    </xsl:attribute>
  </xsl:if>
</xsl:template>



<!-- table formatting properties -->

<xsl:attribute-set name="table.data" ><!-- use for overall table default characteristics -->
  <!--xsl:attribute name="table-layout">fixed</xsl:attribute-->
  <!--xsl:attribute name="inline-progression-dimension">auto</xsl:attribute-->
  <xsl:attribute name="space-before">10pt</xsl:attribute>
  <xsl:attribute name="space-after">10pt</xsl:attribute>
  <xsl:attribute name="background-color">white</xsl:attribute>
  <!--xsl:attribute name="start-indent">inherit</xsl:attribute-->
</xsl:attribute-set>

<xsl:attribute-set name="table.data.caption" >
  <xsl:attribute name="start-indent">inherit</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="table.data.tbody" >
  <xsl:attribute name="background-color">white</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="table.data.th" >
  <xsl:attribute name="color">black</xsl:attribute>
  <xsl:attribute name="background-color">silver</xsl:attribute>
  <xsl:attribute name="padding">2pt</xsl:attribute>
  <xsl:attribute name="text-align">center</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="table.data.tf" >
  <xsl:attribute name="color">blue</xsl:attribute>
  <xsl:attribute name="padding">2pt</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="table.data.td" >
  <xsl:attribute name="background-color">grey</xsl:attribute>
  <xsl:attribute name="padding">2pt</xsl:attribute>
</xsl:attribute-set>


<!-- separate the frame attributes into here; include them conditionally -->
<xsl:attribute-set name="frameall">
  <xsl:attribute name="border-style">solid</xsl:attribute>
  <xsl:attribute name="border-width">1pt</xsl:attribute>
  <xsl:attribute name="border-color">black</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="frametop">
  <xsl:attribute name="border-top-style">solid</xsl:attribute>
  <xsl:attribute name="border-top-width">1pt</xsl:attribute>
  <xsl:attribute name="border-top-color">black</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="framebottom">
  <xsl:attribute name="border-bottom-style">solid</xsl:attribute>
  <xsl:attribute name="border-bottom-width">1pt</xsl:attribute>
  <xsl:attribute name="border-bottom-color">black</xsl:attribute>
</xsl:attribute-set>


<!-- table support based on examples in the XSL spec -->
  <!-- see original support comments in the XSL spec, source of this fragment -->

<xsl:template name="calc.column.width">
  <xsl:param name="colwidth">1*</xsl:param>
  <!-- Ok, the colwidth could have any one of the following forms: -->
  <!--        1*       = proportional width -->
  <!--     1unit       = 1.0 units wide -->
  <!--         1       = 1pt wide -->
  <!--  1*+1unit       = proportional width + some fixed width -->
  <!--      1*+1       = proportional width + some fixed width -->

  <!-- If it has a proportional width, translate it to XSL -->
  <xsl:if test="contains($colwidth, '*')"><!-- modified to handle "*" as input -->
    <xsl:variable name="colfactor">
      <xsl:value-of select="substring-before($colwidth, '*')"/>
    </xsl:variable>
    <xsl:text>proportional-column-width(</xsl:text>
      <xsl:choose>
        <xsl:when test="not($colfactor = '')">
         <xsl:value-of select="$colfactor"/>
        </xsl:when>
        <xsl:otherwise>1</xsl:otherwise>
      </xsl:choose>
    <xsl:text>)</xsl:text>
  </xsl:if>


  <!-- Now get the non-proportional part of the specification -->
  <xsl:variable name="width-units">
    <xsl:choose>
      <xsl:when test="contains($colwidth, '*')">
        <xsl:value-of

             select="normalize-space(substring-after($colwidth, '*'))"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="normalize-space($colwidth)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- Now the width-units could have any one of the following forms: -->
  <!--                 = <empty string> -->
  <!--     1unit       = 1.0 units wide -->
  <!--         1       = 1pt wide -->

  <!-- with an optional leading sign -->

  <!-- Get the width part by blanking out the units part and discarding -->
  <!-- white space. -->
  <xsl:variable name="width"
       select="normalize-space(translate($width-units,
                          '+-0123456789.abcdefghijklmnopqrstuvwxyz',
                          '+-0123456789.'))"/>

  <!-- Get the units part by blanking out the width part and discarding -->
  <!-- white space. -->
  <xsl:variable name="units"
       select="normalize-space(translate($width-units,
                          'abcdefghijklmnopqrstuvwxyz+-0123456789.',
                          'abcdefghijklmnopqrstuvwxyz'))"/>

  <!-- Output the width -->
  <xsl:value-of select="$width"/>

  <!-- Output the units, translated appropriately -->
  <xsl:choose>
    <xsl:when test="$units = 'pi'">pc</xsl:when>
    <xsl:when test="$units = '' and $width != ''">pt</xsl:when>
    <xsl:otherwise><xsl:value-of select="$units"/></xsl:otherwise>
  </xsl:choose>
</xsl:template>


<xsl:template name="xcalc.column.width">
  <!-- see original support comments in the XSL spec, source of this fragment -->
  <xsl:param name="colwidth">1*</xsl:param>

  <!-- Ok, the colwidth could have any one of the following forms: -->
  <!--        1*       = proportional width -->
  <!--     1unit       = 1.0 units wide -->
  <!--         1       = 1pt wide -->
  <!--  1*+1unit       = proportional width + some fixed width -->
  <!--      1*+1       = proportional width + some fixed width -->

  <!-- If it has a proportional width, translate it to XSL -->
  <xsl:if test="contains($colwidth, '*')">
    <xsl:variable name="colfactor">
      <xsl:value-of select="substring-before($colwidth, '*')"/>
    </xsl:variable>
    <xsl:text>proportional-column-width(</xsl:text>
      <xsl:choose>
        <xsl:when test="not($colfactor = '')">
         <xsl:value-of select="$colfactor"/>
        </xsl:when>
        <xsl:otherwise>1</xsl:otherwise>
      </xsl:choose>
    <xsl:text>)</xsl:text>
  </xsl:if>

  <!-- Now get the non-proportional part of the specification -->
  <xsl:variable name="width-units">
    <xsl:choose>
      <xsl:when test="contains($colwidth, '*')">
        <xsl:value-of
             select="normalize-space(substring-after($colwidth, '*'))"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="normalize-space($colwidth)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- Now the width-units could have any one of the following forms: -->
  <!--                 = <empty string> -->
  <!--     1unit       = 1.0 units wide -->
  <!--         1       = 1pt wide -->
  <!-- with an optional leading sign -->

  <!-- Get the width part by blanking out the units part and discarding -->
  <!-- whitespace. -->
  <xsl:variable name="width"
       select="normalize-space(translate($width-units,
                                         '+-0123456789.abcdefghijklmnopqrstuvwxyz',
                                         '+-0123456789.'))"/>

  <!-- Get the units part by blanking out the width part and discarding -->
  <!-- whitespace. -->
  <xsl:variable name="units"
       select="normalize-space(translate($width-units,
                                         'abcdefghijklmnopqrstuvwxyz+-0123456789.',
                                         'abcdefghijklmnopqrstuvwxyz'))"/>

  <!-- Output the width -->
  <xsl:value-of select="$width"/>

  <!-- Output the units, translated appropriately -->
  <xsl:choose>
    <xsl:when test="$units = 'pi'">pc</xsl:when>
    <xsl:when test="$units = '' and $width != ''">pt</xsl:when>
    <xsl:otherwise><xsl:value-of select="$units"/></xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- end of table section -->


<!-- =================== end of element rules ====================== -->




<!-- =================== start of processors  ====================== -->


<!--  COMMON ATTRIBUTE PROCESSORS  -->

<!-- If the element has an ID, copy it through as an anchor. -->
<xsl:template match="@id">
<!-- Filler for A-name anchors (empty links)-->
<xsl:variable name="afill"></xsl:variable>
  <fo:inline>
    <xsl:attribute name="id">
      <xsl:if test="ancestor::*[contains(@class,' topic/body ')]">
       <xsl:value-of select="ancestor::*[contains(@class,' topic/body ')]/parent::*/@id"/><xsl:text>__</xsl:text>
      </xsl:if>
      <xsl:value-of select="."/>
    </xsl:attribute>
  <xsl:value-of select="$afill"/></fo:inline>
</xsl:template>


<!-- Generate an ID to use for generated toc. -->
<xsl:template name="gen-toc-id">
<!-- Filler for A-name anchors (empty links)-->
<xsl:variable name="afill"></xsl:variable>
  <fo:inline id="{generate-id()}"><xsl:value-of select="$afill"/></fo:inline>
</xsl:template>


<!-- Create the class attribute for an element when the DITA element has an outputclass attribute. -->
<xsl:template name="setclass"/>
<xsl:template name="xsetclass">
  <xsl:if test="@outputclass">
     <xsl:attribute name="class"><xsl:value-of select="@outputclass"/></xsl:attribute>
  </xsl:if>
</xsl:template>


<!-- If the element has a compact=yes attribute, assert it in HTML form. -->
<!-- (assumes that no compaction is default) -->
<xsl:template match="@compact">
  <xsl:if test="@compact = 'yes'">
   <xsl:attribute name="compact">compact</xsl:attribute>
  </xsl:if>
</xsl:template>


<!-- Process the scale attribute -->
<xsl:template name="setscale">
  <xsl:if test="@scale">
    <xsl:attribute name="font-size"><xsl:value-of select="@scale"/>%</xsl:attribute>
  </xsl:if>
</xsl:template>


<!--  CONREF (content fetched by id from conrefed element)  -->

<xsl:template match="*[@conref]" priority="10">
  <!--get element local name, parent topic's domains, and then file name, topic id, element id from conref value-->
  <xsl:variable name="element">
    <xsl:value-of select="local-name(.)"/>
  </xsl:variable>
  <xsl:variable name="domains">
    <!--xsl:value-of select="ancestor::*[contains(@class, ' topic/body ')]/parent::*/@domains"/-->
    <xsl:value-of select="ancestor-or-self::*[contains(@class, ' topic/topic ')][1]/@domains"/>
  </xsl:variable>
  <xsl:variable name="file">
    <xsl:choose>
       <xsl:when test="contains(@conref,'#')"><xsl:value-of select="$WORKDIR"/><xsl:value-of select="substring-before(@conref,'#')"/></xsl:when>
       <xsl:otherwise><xsl:value-of select="$WORKDIR"/><xsl:value-of select="@conref"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="topicid">
    <xsl:choose>
       <xsl:when test="contains(@conref,'#') and contains(substring-after(@conref,'#'),'/')"><xsl:value-of select="substring-before(substring-after(@conref,'#'),'/')"/></xsl:when>
       <xsl:when test="contains(@conref,'#')"><xsl:value-of select="substring-after(@conref,'#')"/></xsl:when>
       <xsl:otherwise>none</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="elemid">
    <xsl:choose>
     <xsl:when test="contains(@conref,'#') and contains(substring-after(@conref,'#'),'/')"><xsl:value-of select="substring-after(substring-after(@conref,'#'),'/')"/></xsl:when>
     <xsl:otherwise>none</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="topicpos">
    <xsl:choose>
       <xsl:when test="starts-with(@conref,'#')">samefile</xsl:when>
       <xsl:when test="contains(@conref,'#')">otherfile</xsl:when>
       <xsl:otherwise>firstinfile</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!--xsl:variable name="DBG">no</xsl:variable-->
  <xsl:if test="$DBG='yes'">
    <xsl:call-template name="output-message">
      <xsl:with-param name="msg">Conref trace: </xsl:with-param>
      <xsl:with-param name="msgnum">006</xsl:with-param>
      <xsl:with-param name="msgsev">I</xsl:with-param>
    </xsl:call-template>
    <xsl:message> Element: <xsl:value-of select="name()"/>:<xsl:value-of select="$element"/></xsl:message>
    <xsl:message> Domains: <xsl:value-of select="$domains"/></xsl:message>
    <xsl:message> Working dir: <xsl:value-of select="$WORKDIR"/></xsl:message>
    <xsl:message> File: <xsl:value-of select="$file"/></xsl:message>
    <xsl:message> Topicid: <xsl:value-of select="$topicid"/></xsl:message>
    <xsl:message> Elemid: <xsl:value-of select="$elemid"/></xsl:message>
    <xsl:message> Topic location: <xsl:value-of select="$topicpos"/></xsl:message>
  </xsl:if>

  <xsl:choose>
     <xsl:when test="contains(substring-after(@conref,'#'),'/')"><!--targetting an element inside a topic-->
       <xsl:choose>
          <xsl:when test="$topicpos='samefile'">
            <xsl:apply-templates select="//*[contains(@class, ' topic/topic ')][@id=$topicid][@domains=$domains]/*[contains(@class, ' topic/body ')]//*[local-name()=$element][@id=$elemid]"/>
          </xsl:when>
          <xsl:when test="$topicpos='otherfile'">
            <xsl:apply-templates select="document($file,/)//*[contains(@class, ' topic/topic ')][@id=$topicid][@domains=$domains]/*[contains(@class, ' topic/body ')]//*[local-name()=$element][@id=$elemid]"/>
          </xsl:when>
          <xsl:otherwise/><!-- error msg -->
       </xsl:choose>
     </xsl:when>
     <xsl:otherwise><!--assume targetting a topic-->
       <xsl:choose>
          <xsl:when test="$topicpos='samefile'">
            <xsl:apply-templates select="//*[contains(@class, ' topic/topic ')][@id=$topicid][@domains=$domains][local-name()=$element]"/>
          </xsl:when>
          <xsl:when test="$topicpos='otherfile'">
            <xsl:apply-templates select="document($file,/)//*[contains(@class, ' topic/topic ')][@id=$topicid][@domains=$domains][local-name()=$element]"/>
          </xsl:when>
          <xsl:when test="$topicpos='firstinfile'">
            <xsl:apply-templates select="document($file,/)//*[contains(@class, ' topic/topic ')][1][@domains=$domains][local-name()=$element]"/>
          </xsl:when>
          <xsl:otherwise/><!-- error msg -->
       </xsl:choose>
     </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- SCRIPT SUPPORT -->

<xsl:template name="script-sample">
<!-- not applicable for FO output -->
</xsl:template>



<!--  NAMED TEMPLATES (call by name, only)  -->

<xsl:template name="gen-att-label">
  <xsl:if test="@spectitle">
    <fo:block margin-bottom="0" font-weight="bold">
    <xsl:value-of select="@spectitle"/></fo:block>
  </xsl:if>
</xsl:template>


<!-- named templates that can be used anywhere -->

<!-- this replaces newlines with the BR element, forcing non-concatenation even in flow contexts -->
<xsl:template name="br-replace">
  <xsl:param name="word"/>
<!-- capture an actual newline within the xsl:text element -->
  <xsl:variable name="cr"><xsl:text>
</xsl:text></xsl:variable>
  <xsl:choose>
    <xsl:when test="contains($word,$cr)">
       <xsl:value-of select="substring-before($word,$cr)"/>
       <!--br class="br"/-->
       <xsl:call-template name="br-replace">
         <xsl:with-param name="word" select="substring-after($word,$cr)"/>
       </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$word"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- diagnostic: call this to generate a path-like view of an element's ancestry! -->
<xsl:template name="breadcrumbs">
<xsl:variable name="full-path">
  <xsl:for-each select="ancestor-or-self::*">
    <xsl:value-of select="concat('/',name())"/>
  </xsl:for-each>
</xsl:variable>
<fo:block font-weight="bold"><xsl:value-of select="$full-path"/></fo:block>
</xsl:template>


<!-- the following named templates generate inline content for the delivery context -->


<!-- generate null filler if the phrase is evidently empty -->
<xsl:template name="apply-for-phrases">
  <xsl:choose>
    <xsl:when test="not(text()[normalize-space(.)] | *)"><xsl:comment>null</xsl:comment></xsl:when>
    <xsl:otherwise><xsl:apply-templates/></xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- named templates for labels and titles related to topic structures -->

<xsl:template name="get-title"><!-- get fully-processed title content by whatever mechanism -->
  <xsl:choose>
   <!-- add keycol here once implemented -->
   <xsl:when test="@spectitle">
     <xsl:value-of select="@spectitle"/>
   </xsl:when>
   <xsl:otherwise>
    <xsl:apply-templates/> <!-- select="title|*[contains(@class,' topic/title ')]"/-->
   </xsl:otherwise>
  </xsl:choose>
</xsl:template>



<xsl:template name="get-sect-heading">
     <xsl:choose>
      <!-- replace with keyref once implemented -->
      <xsl:when test="@spectitle">
        <xsl:value-of select="@spectitle"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="title"/>
      </xsl:otherwise>
     </xsl:choose>
</xsl:template>


<xsl:template name="sect-heading">
  <xsl:param name="deftitle" select="."/> <!-- get param by reference -->
  <xsl:variable name="heading">
     <xsl:choose>
      <xsl:when test="*[contains(@class,' topic/title ')]">
        <xsl:value-of select="*[contains(@class,' topic/title ')]"/>
        <xsl:if test="*[contains(@class,' topic/title ')][2]">
         <xsl:call-template name="output-message">
           <xsl:with-param name="msg">More than one title element in a section. Using the first one for the section's title.</xsl:with-param>
           <xsl:with-param name="msgnum">007</xsl:with-param>
           <xsl:with-param name="msgsev">W</xsl:with-param>
         </xsl:call-template>
      </xsl:if>
      </xsl:when>
      <xsl:when test="@spectitle">
        <xsl:value-of select="@spectitle"/>
      </xsl:when>
      <xsl:otherwise/>
     </xsl:choose>
  </xsl:variable>

  <!-- based on graceful defaults, build an appropriate section-level heading -->
  <xsl:choose>
    <xsl:when test="not($heading='')">
      <xsl:if test="normalize-space($heading)=''">
        <!-- hack: a title with whitespace ALWAYS overrides as null -->
        <xsl:comment>no heading</xsl:comment>
      </xsl:if>
      <fo:block font-weight="bold"><!--xsl:call-template name="proc-ing"/--><xsl:value-of select="$heading"/></fo:block>
    </xsl:when>
    <xsl:when test="$deftitle">
      <fo:block font-weight="bold"><xsl:value-of select="$deftitle"/></fo:block>
    </xsl:when>
    <xsl:otherwise><!--br /--><!-- no heading title, output section starting with a break --></xsl:otherwise>
  </xsl:choose>
</xsl:template>




<!-- render any contained footnotes as endnotes.  Links back to reference point -->
<!-- TBD: test to see if the footnote is in a draft-comment or required-cleanup context -->
<!--      if so, need to filter them out! -->
<!-- Basic reuse issue: if ANY addressable content within these elements IS referenced, it will
      be pulled and instanced. -->

<!-- not included:
-  gen-endnotes
-  gen-toc
-  gen-ptoc
-  gen-sect-ptoc
as these involved a lot of HTML to clean up for FO
-->  



<!-- basis for string localization -->

<xsl:template name="xgetString">
  <xsl:param name="stringName"/>
  <xsl:value-of select="$stringName"/>
</xsl:template>

<!-- language support (from an algorithm described in XSL FAQ) -->
<!--
        given the info needed to produce a set of candidates ($str),
           pick the best of the bunch:
           1. $str[lang($Lang)][1]
           2. $str[lang($PrimaryLang)][1]
           3. $str[1]
           4. if not($str) then issue warning to STDERR
   - -->
<xsl:template name="getString">
  <xsl:param name="stringName"/>
  <xsl:param name="stringFile" select="$StringFile"/>
  <xsl:variable name="str" 
     select="$stringFile/strings/str[@name=$stringName]"/>
  <xsl:choose>
    <xsl:when test="$str[lang($Lang)]">
      <xsl:value-of select="$str[lang($Lang)][1]"/>
    </xsl:when>
    <xsl:when test="$str[lang($PrimaryLang)]">
      <xsl:value-of select="$str[lang($PrimaryLang)][1]"/>
    </xsl:when>
    <xsl:when test="$str">
      <xsl:value-of select="$str[1]"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:message terminate="no">
        <xsl:text>Warning: no string named '</xsl:text>
        <xsl:value-of select="$stringName"/>
        <xsl:text>' found.</xsl:text>
      </xsl:message>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>




<!--  SETTINGS  -->
<xsl:variable name="trace">no</xsl:variable> <!--set string to 'yes' to turn on trace -->

<!-- set up keys based on xref's "type" attribute: %info-types;|hd|fig|table|li|fn -->
<xsl:key name="topic" match="topic" use="@id"/> <!-- uses "title" -->
<xsl:key name="fig"   match="fig"   use="@id"/> <!-- uses "title" -->
<xsl:key name="table" match="table" use="@id"/> <!-- uses "title" -->

<xsl:key name="li"    match="li"    use="@id"/> <!-- uses "?" -->
<xsl:key name="fn"    match="fn"    use="@id"/> <!-- uses "callout?" -->



<!-- CONTROL PARAMETERS -->
<!-- offset -->
<xsl:param name="basic-start-indent">72pt</xsl:param>
<xsl:param name="basic-end-indent">24pt</xsl:param>
<xsl:param name="basic-first-indent">84pt</xsl:param>


<!-- ========== FORMATTER DECLARATIONS AND GLOBALS ========== -->

<!-- set up CSS-like attribute sets -->
<!-- tempted to use the shorthand form? if so, beware of "silent overrides" -->

<xsl:attribute-set name="topic" >
  <xsl:attribute name="font-family">Helvetica</xsl:attribute>
  <xsl:attribute name="font-size">10pt</xsl:attribute>
</xsl:attribute-set>

<!-- set up common attributes for all page definitions -->
<xsl:attribute-set name="common-grid">
  <xsl:attribute name="page-width">51pc</xsl:attribute> <!-- A4: 210mm -->
  <xsl:attribute name="page-height">66pc</xsl:attribute> <!-- A4: 297mm -->
  <xsl:attribute name="margin-top">3pc</xsl:attribute>
  <xsl:attribute name="margin-bottom">3pc</xsl:attribute>
  <xsl:attribute name="margin-left">6pc</xsl:attribute>
  <xsl:attribute name="margin-right">6pc</xsl:attribute>
</xsl:attribute-set>

<!-- add line-height for multi-line headings? -->

<xsl:attribute-set name="h0.title" >
  <xsl:attribute name="break-before">page</xsl:attribute>
  <xsl:attribute name="margin-top">0pc</xsl:attribute>
  <xsl:attribute name="margin-bottom">1.4pc</xsl:attribute>
  <xsl:attribute name="font-size">16pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="topictitle1" >
  <xsl:attribute name="break-before">page</xsl:attribute>
  <xsl:attribute name="margin-top">0pc</xsl:attribute>
  <xsl:attribute name="margin-bottom">1.4pc</xsl:attribute>
  <xsl:attribute name="font-size">16pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="topictitle2" >
  <xsl:attribute name="margin-top">1pc</xsl:attribute>
  <xsl:attribute name="margin-bottom">5pt</xsl:attribute>
  <xsl:attribute name="font-size">14pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="topictitle3" >
  <xsl:attribute name="margin-top">1pc</xsl:attribute>
  <xsl:attribute name="margin-bottom">2pt</xsl:attribute>
  <xsl:attribute name="font-size">14pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="topictitle4" >
  <xsl:attribute name="font-size">9pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
  <xsl:attribute name="keep-with-next.within-page">always</xsl:attribute>
</xsl:attribute-set>


<xsl:attribute-set name="topictitle5" >
  <xsl:attribute name="font-size">11pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
  <xsl:attribute name="keep-with-next.within-page">always</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="topictitle6" >
  <xsl:attribute name="font-size">11pt</xsl:attribute>
  <xsl:attribute name="font-style">italic</xsl:attribute>
  <xsl:attribute name="keep-with-next.within-page">always</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
</xsl:attribute-set>


<xsl:attribute-set name="section.title" >
  <xsl:attribute name="space-before">1em</xsl:attribute>
  <xsl:attribute name="color">black</xsl:attribute>
  <xsl:attribute name="font-size">11pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
  <xsl:attribute name="keep-with-next.within-page">always</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="unused.section.title">
  <xsl:attribute name="space-before">1em</xsl:attribute>
  <xsl:attribute name="space-before.precedence">1</xsl:attribute>
  <xsl:attribute name="space-after">1em</xsl:attribute>
  <xsl:attribute name="keep-with-next.within-column">always</xsl:attribute>
  <xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute>
</xsl:attribute-set>


<!-- p-->
<xsl:attribute-set name="p">
  <xsl:attribute name="text-indent">0em</xsl:attribute>
  <xsl:attribute name="space-before">0.6em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <!--xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute-->
  <!--xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute-->
  <!--xsl:attribute name="text-align">justify</xsl:attribute-->
</xsl:attribute-set>

<!-- p-->
<xsl:attribute-set name="top.p">
  <xsl:attribute name="text-indent">0em</xsl:attribute>
  <xsl:attribute name="space-before">0.6em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <!--xsl:attribute name="text-align">justify</xsl:attribute-->
</xsl:attribute-set>

<!-- p-->
<xsl:attribute-set name="divlike.p">
  <xsl:attribute name="text-indent">0em</xsl:attribute>
  <xsl:attribute name="space-before">0em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <!--xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute-->
  <!--xsl:attribute name="text-align">justify</xsl:attribute-->
</xsl:attribute-set>

<xsl:attribute-set name="lq" >
  <!--xsl:attribute name="background-color">antiquewhite</xsl:attribute-->
  <!--xsl:attribute name="font-style">italic</xsl:attribute-->
  <xsl:attribute name="space-before">10pt</xsl:attribute>
  <xsl:attribute name="space-after">10pt</xsl:attribute>
  <xsl:attribute name="padding-left">2pt</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
  <xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute>
  <xsl:attribute name="text-align">left</xsl:attribute>
  <xsl:attribute name="border-style">solid</xsl:attribute>
  <xsl:attribute name="border-color">black</xsl:attribute>
  <xsl:attribute name="border-width">thin</xsl:attribute>
</xsl:attribute-set>


<!-- footnote-->
<xsl:attribute-set name="footnote">
  <xsl:attribute name="font-family">Helvetica</xsl:attribute>
  <xsl:attribute name="font-size">8pt</xsl:attribute>
  <xsl:attribute name="line-height">10pt</xsl:attribute>
  <xsl:attribute name="text-indent">0em</xsl:attribute>
  <xsl:attribute name="space-before">0em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <xsl:attribute name="start-indent">0in</xsl:attribute>
  <!--xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute-->
  <xsl:attribute name="text-align">justify</xsl:attribute>
</xsl:attribute-set>


<!-- note -->
<xsl:attribute-set name="note" >
  <xsl:attribute name="background-color">antiquewhite</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
  <xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute>
</xsl:attribute-set>


<!-- ul-->
<xsl:attribute-set name="ul">
  <xsl:attribute name="text-indent">0em</xsl:attribute>
  <xsl:attribute name="space-before">0.6em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="dd.p">
  <xsl:attribute name="text-indent">0em</xsl:attribute>
</xsl:attribute-set>


<xsl:attribute-set name="dt" >
  <xsl:attribute name="font-weight">bold</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
  <xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="dd" >
  <xsl:attribute name="space-before">0.3em</xsl:attribute>
  <xsl:attribute name="space-after">0.5em</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/> + 4em</xsl:attribute>
  <xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="pre">
  <!--xsl:attribute name="start-indent">inherit</xsl:attribute-->
  <xsl:attribute name="padding-left">2pt</xsl:attribute>
  <xsl:attribute name="space-before">0.6em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <xsl:attribute name="white-space">pre</xsl:attribute>
  <xsl:attribute name="white-space-collapse">false</xsl:attribute>
  <xsl:attribute name="wrap-option">wrap</xsl:attribute>
  <xsl:attribute name="background-color">#e0e0e0</xsl:attribute>


  <xsl:attribute name="font-family">Courier</xsl:attribute>
  <xsl:attribute name="line-height">106%</xsl:attribute>
  <xsl:attribute name="font-size">9pt</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="lines">
  <xsl:attribute name="space-before">0.6em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <xsl:attribute name="white-space">pre</xsl:attribute>
  <xsl:attribute name="white-space-collapse">false</xsl:attribute>
  <xsl:attribute name="wrap-option">wrap</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="fig">
  <xsl:attribute name="space-before">0.6em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
</xsl:attribute-set>


<xsl:attribute-set name="figure.title" > <!-- unused-->
  <xsl:attribute name="text-align">center</xsl:attribute>
  <xsl:attribute name="space-before">3pt</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
  <xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute>
</xsl:attribute-set>






<!--  "FORMAT" GLOBAL DECLARATIONS  -->

<xsl:variable name="fmt-fig-lbl-loc">over</xsl:variable><!-- values: over, under -->
<xsl:variable name="fmt-tbl-lbl-loc">over</xsl:variable><!-- values: over, under -->
<xsl:variable name="link-top-section">no</xsl:variable><!-- values: yes, no (or any not "yes") -->
<xsl:variable name="do-place-ing">no</xsl:variable><!-- values: yes, no (or any not "yes") -->

<!-- this value should be created in a named template since it needs to increment per call -->
<!-- for now, the static value is null until we can redo this intent -->
<xsl:variable name="fig-pfx-txt"></xsl:variable>
<xsl:variable name="xfig-pfx-txt">
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Figure'"/>
      </xsl:call-template> <xsl:number level="any" count="label" from="/" />
 of <xsl:value-of select="count(//fig/label)"/>. </xsl:variable><!-- values: '' or custom design -->
<xsl:variable name="tbl-pfx-txt"></xsl:variable>



<!--  "FORMAT" MACROS  -->
<!--
 | These macros support globally-defined formatting constants for
 | document content.  Some elements have attributes that permit local
 | control of formatting; such logic is part of the pertinent template rule.
 +-->

<xsl:template name="place-tbl-width">
<xsl:variable name="twidth-fixed">100%</xsl:variable>
  <xsl:if test="$twidth-fixed != ''">
    <xsl:attribute name="width"><xsl:value-of select="$twidth-fixed"/></xsl:attribute>
  </xsl:if>
</xsl:template>


<xsl:template name="place-tbl-lbl">
<xsl:variable name="tbl-count">                  <!-- Number of table/title's before this one -->
 <xsl:number count="*/table/title" level="any"/>
</xsl:variable>
<xsl:variable name="tbl-count-actual">           <!-- Number of table/title's including this one -->
 <xsl:choose>
   <xsl:when test="not($tbl-count&gt;0) and not($tbl-count=0) and not($tbl-count&lt;0)">1</xsl:when>
   <xsl:otherwise><xsl:value-of select="$tbl-count+1"/></xsl:otherwise>
 </xsl:choose>
</xsl:variable>
  <xsl:choose>
    <xsl:when test="*[contains(@class,' topic/title ')]">
      <fo:block><fo:inline font-weight="bold">
        <xsl:call-template name="getString">
         <xsl:with-param name="stringName" select="'Table'"/>
        </xsl:call-template><xsl:text> </xsl:text><xsl:value-of select="$tbl-count-actual"/>.<xsl:text> </xsl:text>
        <xsl:apply-templates select="*[contains(@class,' topic/title ')]" mode="exhibittitle"/>
      </fo:inline>
      <xsl:if test="*[contains(@class,' topic/desc ')]">
        <xsl:text>. </xsl:text><xsl:apply-templates select="*[contains(@class,' topic/desc ')]" mode="exhibitdesc"/>
      </xsl:if>
      </fo:block>
    </xsl:when>
    <xsl:when test="*[contains(@class,' topic/desc ')]">
      <fo:block>****<xsl:value-of select="*[contains(@class,' topic/desc ')]"/></fo:block>
    </xsl:when>
  </xsl:choose>
</xsl:template>


<xsl:template name="place-fig-lbl">
<xsl:variable name="fig-count">                 <!-- Number of fig/title's before this one -->
 <xsl:number count="*/fig/title" level="any"/>
</xsl:variable>
<xsl:variable name="fig-count-actual">          <!-- Number of fig/title's including this one -->
 <xsl:choose>
   <xsl:when test="not($fig-count&gt;0) and not($fig-count=0) and not($fig-count&lt;0)">1</xsl:when>
   <xsl:otherwise><xsl:value-of select="$fig-count+1"/></xsl:otherwise>
 </xsl:choose>
</xsl:variable>
  <xsl:choose>
    <xsl:when test="*[contains(@class,' topic/title ')]">
      <fo:block><fo:inline font-weight="bold">
        <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Figure'"/>
        </xsl:call-template><xsl:text> </xsl:text><xsl:value-of select="$fig-count-actual"/>.<xsl:text> </xsl:text>
        <xsl:apply-templates select="*[contains(@class,' topic/title ')]" mode="exhibittitle"/>
      </fo:inline>
      <xsl:if test="desc">
        <xsl:text>. </xsl:text><xsl:apply-templates select="*[contains(@class,' topic/desc ')]" mode="exhibitdesc"/>
      </xsl:if>
      </fo:block>
    </xsl:when>
    <xsl:when test="*[contains(@class, ' topic/desc ')]">
      <fo:block>****<xsl:value-of select="*[contains(@class,' topic/desc ')]"/></fo:block>
    </xsl:when>
  </xsl:choose>
</xsl:template>



<xsl:template name="proc-ing">
  <xsl:if test="$do-place-ing = 'yes'"> <!-- set in a global variable, as with label placement, etc. -->
    <fo:external-graphic src="url(image/tip-ing.jpg)"/> <!-- this should be an xsl:choose with the approved list and a selection method-->
    <!-- add any other required positioning controls, if needed, but must be valid in the location
         from which the call to this template was made -->
    &nbsp;
  </xsl:if>
</xsl:template>

<!-- =================== end of processors  ====================== -->




<!-- =================== start of diagnostics  ====================== -->


<xsl:template name="output-message">
  <xsl:param name="msg" select="***"/>
  <xsl:param name="msgnum" select="000"/>
  <xsl:param name="msgsev" select="I"/>
  <xsl:variable name="localclass"><xsl:value-of select="@class"/></xsl:variable>
  <xsl:variable name="debugloc">
   <!-- DocID = test.dita, Element = searchtitle:1 -->
   <xsl:text>DocID = </xsl:text><xsl:value-of select="generate-id(/)"/><xsl:text>, </xsl:text>
   <xsl:text>Element = </xsl:text><xsl:value-of select="name()"/>:<xsl:value-of select="count(preceding::*[@class=$localclass])+count(ancestor-or-self::*[@class=$localclass])"/>
  </xsl:variable>
  <xsl:message><xsl:text>------------------------------------------------------------------
DITA</xsl:text><xsl:value-of select="$msgnum"/>
    <xsl:choose>
      <xsl:when test="$msgsev='I'">I Informational: </xsl:when>
      <xsl:when test="$msgsev='W'">W Warning: </xsl:when>
      <xsl:when test="$msgsev='E'">E Error: </xsl:when>

      <xsl:when test="$msgsev='F'">F Fatal: </xsl:when>
      <xsl:otherwise>I Informational: </xsl:otherwise>
    </xsl:choose>
    <xsl:text> (</xsl:text><xsl:value-of select="$debugloc"/><xsl:text>)
</xsl:text><xsl:value-of select="$msg"/><xsl:text>
</xsl:text>
  </xsl:message>
</xsl:template>

<!-- =================== end of diagnostics  ====================== -->




<!-- =================== start of override stubs ====================== -->

<!--  STUBS FOR USER PROVIDED OVERRIDE EXTENSIONS  -->

<xsl:template name="gen-user-header">
  <!-- to customize: copy this to your override transform, add whatever content you want! -->
  <!-- for example, to display logos, search/nav widgets, etc. -->
</xsl:template>

<xsl:template name="gen-user-footer">
  <!-- to customize: copy this to your override transform, add whatever content you want! -->
  <!-- for example, to display compliances for: XHTML, accessibility, content ratings, etc. -->
</xsl:template>

<xsl:template name="gen-user-sidetoc">
  <!-- to customize: copy this to your override transform, add whatever content you want! -->
  <!-- Common implementations use a table with align=right to place generated content
       adjacent to the start of the body content -->
</xsl:template>

<xsl:template name="gen-user-scripts">
  <!-- to customize: copy this to your override transform, add whatever content you want! -->
  <!-- see (or enable) the named template "script-sample" for an example -->
  <!--xsl:call-template name="script-sample"/-->
</xsl:template>

<xsl:template name="gen-user-styles">
  <!-- to customize: copy this to your override transform, add whatever content you want! -->
</xsl:template>


<xsl:template name="gen-user-panel-title-pfx">
  <!-- to customize: copy this to your override transform, add whatever content you want! -->
  <!-- Generate content based on run-time parameter value, with local logic here -->
  <!-- This is overrideable -->
</xsl:template>

<xsl:template name="gen-main-panel-title">
  <!-- use the searchtitle unless there's no value - else use title -->
  <xsl:variable name="schtitle"><xsl:value-of select="/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/titlealts ')]/*[contains(@class,' topic/searchtitle ')]"/></xsl:variable>
  <xsl:variable name="ditaschtitle"><xsl:value-of select="/dita/*[contains(@class,' topic/topic ')][1]/*[contains(@class,' topic/titlealts ')]/*[contains(@class,' topic/searchtitle ')]"/></xsl:variable>
  <xsl:variable name="maintitle"><xsl:value-of select="/*[contains(@class,' topic/topic ')]/*[contains(@class,' topic/title ')]"/></xsl:variable>
  <xsl:variable name="ditamaintitle"><xsl:value-of select="/dita/*[contains(@class,' topic/topic ')][1]/*[contains(@class,' topic/title ')]"/></xsl:variable>
  <xsl:choose>
    <xsl:when test="string-length($schtitle)>'0'"><xsl:value-of select="$schtitle"/></xsl:when>
    <xsl:when test="string-length($ditaschtitle)>'0'"><xsl:value-of select="$ditaschtitle"/></xsl:when>
    <xsl:when test="string-length($maintitle)>'0'"><xsl:value-of select="$maintitle"/></xsl:when>
    <xsl:when test="string-length($ditamaintitle)>'0'"><xsl:value-of select="$ditamaintitle"/></xsl:when>
    <xsl:otherwise>
      <xsl:text>***</xsl:text>
      <xsl:call-template name="output-message">
        <xsl:with-param name="msg">No title specified; using "***".</xsl:with-param>
        <xsl:with-param name="msgnum">009</xsl:with-param>
        <xsl:with-param name="msgsev">W</xsl:with-param>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="gen-user-metadata"/>
<!-- not applicable yet within FO -->

<!-- =================== end of override stubs ====================== -->



<!-- =================== DEFAULT PAGE LAYOUT ====================== -->

<xsl:template name="chapter-setup">
<!-- Newline character (capture the native file newline) -->
<xsl:variable name="newline">
</xsl:variable>

<fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">



  <fo:layout-master-set>

    <fo:simple-page-master master-name="cover"
      page-height="792pt"     page-width="612pt"
      margin-top="36pt"       margin-bottom="36pt"
      margin-left="36pt"      margin-right="36pt">
      <fo:region-body margin-top="120pt"/>
    </fo:simple-page-master>

    <fo:simple-page-master master-name="leftPage"
      page-height="792pt"     page-width="612pt"
      margin-top="36pt"       margin-bottom="36pt"
      margin-left="72pt"      margin-right="60pt">
      <fo:region-body margin-bottom="36pt" margin-top="24pt" />
      <fo:region-before extent="24pt"/>
      <fo:region-after  extent="36pt"/>
    </fo:simple-page-master>

    <fo:simple-page-master master-name="rightPage"
      page-height="792pt"     page-width="612pt"
      margin-top="36pt"       margin-bottom="36pt"
      margin-left="60pt"      margin-right="72pt">
      <fo:region-body margin-bottom="36pt" margin-top="24pt" />
      <fo:region-before extent="24pt"/>
      <fo:region-after  extent="36pt"/>
    </fo:simple-page-master>

    <fo:simple-page-master master-name="common-page"
      page-height="792pt"     page-width="612pt"
      margin-top="36pt"       margin-bottom="36pt"
      margin-left="60pt"      margin-right="60pt">
      <fo:region-body margin-bottom="36pt" margin-top="24pt" />
      <fo:region-before extent="24pt"/>
      <fo:region-after  extent="36pt"/>
    </fo:simple-page-master>

  </fo:layout-master-set>

  <!--xsl:call-template name="gen-cover-page"-->

  <!-- initiate content processing within basic page "shell" -->
  <fo:page-sequence master-reference="common-page">

  <!-- header -->
  <fo:static-content flow-name="xsl-region-before" font-size="9pt" font-family="Helvetica">
    <fo:block linefeed-treatment="ignore" text-align-last="justify">
      <fo:inline text-align="start">
        <xsl:text>Stub for header content</xsl:text>
      </fo:inline>
      <fo:leader rule-style="solid" leader-length.maximum="100%" leader-length.optimum="60%" leader-length.minimum="10pt" leader-pattern="use-content">&nbsp;</fo:leader>
      <fo:inline text-align="end" font-weight="bold">
        <xsl:text>right side content</xsl:text>
      </fo:inline>
    </fo:block>
    <!--fo:block><fo:leader leader-pattern="rule" leader-length="100%"/></fo:block--> 
  </fo:static-content>


  <!-- footer -->
  <fo:static-content flow-name="xsl-region-after">
    <fo:block linefeed-treatment="ignore" text-align-last="justify">
      <fo:inline text-align="start" font-size="10pt" font-family="Helvetica">
        <xsl:value-of select="//*/*[contains(@class, ' topic/title ')]"/>
      </fo:inline>
      <!--fo:leader rule-style="solid" leader-length.maximum="100%" leader-length.optimum="60%" leader-length.minimum="10pt" leader-pattern="use-content">&nbsp;</fo:leader-->
      <fo:inline text-align="end" font-size="10pt" font-weight="bold"  font-family="Helvetica">
        <xsl:text>Page </xsl:text><fo:page-number/>
      </fo:inline>
    </fo:block>
  </fo:static-content>

    <!-- body -->
    <fo:flow flow-name="xsl-region-body" >
      <fo:block line-height="12pt" font-size="10pt"
                font-family="Helvetica"
                id="page1-1">

         <xsl:apply-templates />
      </fo:block>
   </fo:flow>

  </fo:page-sequence>

</fo:root>
</xsl:template>


<xsl:template name="gen-cover-page">
  <!-- generate an "outside-front cover" page -->
  <fo:page-sequence master-reference="cover">
    <fo:flow flow-name="xsl-region-body" >
      <fo:block text-align="center">
        <fo:block space-after.optimum="40pt"
                  space-before.optimum="40pt"
                  line-height="80pt" font-size="32pt"
                  font-family="Helvetica">
          <xsl:value-of select="/topic/title"/>
        </fo:block>
        <fo:block line-height="50pt" font-size="24pt" font-family="Helvetica">
           <fo:block font-style="italic">
             <xsl:value-of select="*[contains(@class, ' topic/title ')]"/>
           </fo:block>
        </fo:block>
      </fo:block>
    </fo:flow>
  </fo:page-sequence>

  <!-- generate an "inside front cover" page (left side) -->
  <fo:page-sequence master-reference="cover">
    <fo:flow flow-name="xsl-region-body" >
      <fo:block xsl:use-attribute-sets="p" color="purple" text-align="center">[TBD: this page intentionally left blank.]</fo:block>
    </fo:flow>
  </fo:page-sequence>
</xsl:template>




<!-- domains (should be external) -->

</xsl:transform>